// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const SAVE_INT = 10; // in seconds (for recovery)

var gDraftId = null;
var gRecoveryId = null;

var gOverrideParagraphBehavior = false;
// var gHasChanged = false;
var gLastSaved = "";
var gWebService;
var gB;

var gEditor = null;
var gNotificationBox = null;

/*
 * Coop stuff
 */

var faves_coop = Components.classes['@flock.com/singleton;1'].getService(Components.interfaces.flockISingleton).getSingleton('chrome://browser/content/flock/common/load-faves-coop.js').wrappedJSObject;

/*
 * Helpers
 */

function getHoursMinutes() {
    var time = new Date();
    var minutes = time.getMinutes();
    var hours = time.getHours();
    if (hours < 10)
        hours = "0" + hours.toString();
    if (minutes < 10)
        minutes = "0" + minutes.toString();

    return hours+':'+minutes;
}

/*
 * load/close functions
 */

editorOnload = function() {
  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('blog open');

  if (window.arguments) {
    var params = window.arguments[0].QueryInterface(Components.interfaces.nsIDialogParamBlock);
    gDraftId = params.GetString(0);
  }
  else {
    gDraftId = null;
  }

  var title = "";
  var content = "";
  var tags = "";

  // Hide unwanted stuff designed for browser.xul
  $('viewTopbarMenu').setAttribute('hidden', 'true');

  gNotificationBox = $("notificationBox");
  gEditor = new editorWidget($('editordeck'), $('edit'));
  updateLastSaved();

  if (gDraftId) { // Load post
    var draft = flock.blog.service.getDraft(gDraftId);
    if (draft) {
      title = draft.title;
      content = draft.content;
      tags = draft.tags;
    }
  }
  else { // new post
    title = params.GetString(1);
    content = params.GetString(2);
    tags = params.GetString(3);
  }

  if (content && content != "")
    gEditor.setContent(content);
  if (title != "")
    $("flock-blog-title").setAttribute("value", title);
  $("flock-blog-tags").value = tags;

  flock_blogEditorSetTitle();
  window.document.commandDispatcher.updateCommands('focus');

  try { $("flock-blog-title").select(); } catch(e) { }

  if (gDraftId) {
    // We only run it for 'open post', not for 'blog this' before we want the
    // the confirmation dialog to appear after Blog This even if the user didn't touch
    // the post
    updateLastSaved();
  }
  flock_blogEditorSetupEditorCommands();
  
  // autosave for recovery
  var sync = {
    notify: function (timer) {
      saveRecovery();
    }
  }

  window._timer = Components.classes['@mozilla.org/timer;1'].createInstance(Components.interfaces.nsITimer);
  window._timer.initWithCallback(sync, SAVE_INT * 1000, Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
};


editorOnunload = function() {
  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('blog close');
  
}


editorOnclose = function() {
  var close = flock_blogEditorCanClose();
  if (close) {
    if (gRecoveryId)
      flock.blog.service.removeRecovery(gRecoveryId);
    try { 
      gEditor.uninit(); 
    } catch(e) { } 
  }
  return close;
};

/*
 * Toolbar customization
 */

function BrowserCustomizeToolbar()
{
  window.openDialog("chrome://browser/content/flock/blog/customizeToolbar.xul", "CustomizeToolbar",
    "chrome,all,dependent", document.getElementById("shelfEditorToolbox"));
}

/*
 * Function related to saving drafts
 */

function flock_blogEditorCanClose()
{
  var result;
  var strbundle=document.getElementById("editorStringBundle");
  var saveentry=strbundle.getString("flock.blog.post.save.entry");
  var doyouwanttosave=strbundle.getString("flock.blog.post.save.draft");

  if(!hasChanged())
    return true;

  var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
  var flags=promptService.BUTTON_TITLE_SAVE * promptService.BUTTON_POS_0 +
      promptService.BUTTON_TITLE_CANCEL * promptService.BUTTON_POS_1 +
      promptService.BUTTON_TITLE_DONT_SAVE * promptService.BUTTON_POS_2;

  result = promptService.confirmEx(window, saveentry, doyouwanttosave, flags,
       null, null, null, null, {});
  
  switch (result){
    case 0: // Save
        flock_blogEditorSavePostCommand.exec();      
        return true;
    case 1: // Cancel
        return false;
    case 2: // Don't save
        return true;
    default:
        return true;
  }
}


function onClickInEditor(event)
{
  $('edit').contentWindow.focus();
  if (event.rangeParent && (event.rangeParent.nodeName == "BODY")) {
    if (!gEditor.getXHtml().match(/[^\s\n]/))
      gEditor.mNSHTMLEditor.beginningOfDocument();
  }
}


function flock_blogEditorSetTitle() {
    var strbundle = document.getElementById("editorStringBundle");
    var title = strbundle.getString("flock.blog.settitle.message") + " " + $("flock-blog-title").value;
    // window.title = title;
    window.document.title = title;
}


function updateLastSaved() {
  gLastSaved = $("flock-blog-title").value + gEditor.getXHtml() + $('flock-blog-tags').value;
}


function onTitleKeypress(aEvent) {
  // gHasChanged = true;
  switch(aEvent.keyCode) {
    case aEvent.DOM_VK_TAB:
      if (gEditor.viewSource)
        $('editsource').focus();
      else {
        $('edit').contentWindow.focus();
        if (!gEditor.getXHtml().match(/[^\s\n]/))
          gEditor.mNSHTMLEditor.beginningOfDocument();
      }
      aEvent.preventDefault();
      break;
    default:
      ;
    }
};


function hasChanged() {
  return (gLastSaved != $("flock-blog-title").value + gEditor.getXHtml() + $('flock-blog-tags').value);
}


function onEditorKeypress(e) {
  goUpdateCommand("cmd_flock_blogSavePost");
}


function saveRecovery() {
  var html;
  if ($('editordeck').selectedIndex == 0)
    html = gEditor.getXHtml();
  else
    html = $('editsource').value;

  dump("Save recovery "+gRecoveryId+"...\n");
  gRecoveryId = flock.blog.service.saveRecovery(gRecoveryId, $("flock-blog-title").value, html, $('flock-blog-tags').value);
}


var flock_blogEditorSavePostCommand = {
  isEnabled: function() {
    return hasChanged();
  },
  exec: function() {
    var html;
    if ($('editordeck').selectedIndex == 0)
      html = gEditor.getXHtml();
    else
      html = $('editsource').value;

    gDraftId = flock.blog.service.saveDraft(gDraftId, $("flock-blog-title").value, html, $('flock-blog-tags').value);
    updateLastSaved();
    goUpdateCommand("cmd_flock_blogSavePost");
  }
}


var flock_blogEditorOpenPostCommand = {
  isEnabled: function() {
    return true;
  },
  exec: function() {
    window.openDialog("chrome://browser/content/flock/blog/blogOpenPost.xul", "openDraftDialog", "chrome,modal,centerscreen", null);
  }
}

/*
 * Commands 
 */

var gBlogEditCommandController = null;

var flock_blogLink = {
  isEnabled: function() {
    return gEditor.linkEnabled();
  },
  exec: function() {
    gEditor.link();
  }
}

var flock_blogUnlink = {
  isEnabled: function() {
    return gEditor.unlinkEnabled();
  },
  exec: function() {
    gEditor.execCommand('unlink');
  }
}

var flock_blogClose = {
  isEnabled: function() {
    return true;
  },
  exec: function() {
    window.controllers.removeController(gBlogEditCommandController);
    flock_blogEditorSavePostCommand.exec();      
    window._timer.cancel();
    window.close();
  }
}

var flock_blogEditorNewPostCommand = {
  isEnabled: function() {
    return true;
  },
  exec: function() {
    flock.blog.service.openEditor("", "", "");
  }
}

var flock_blogEditorPublishPostCommand = {
  isEnabled: function() {
    return true;
  },
  exec: function() {
     var metrics = Components.classes['@flock.com/metrics-service;1']
       .getService(Components.interfaces.flockIMetricsService);
     metrics.reportCount('blog publish');

    // Save the post
    flock_blogEditorSavePostCommand.exec();      

    // Setup an account if needed
    if(flock.blog.service.accountCount() == 0)
      flock_blogOpenWizard();

    var strbundle = document.getElementById("editorStringBundle");
    var msgtxt = strbundle.getString("flock.blog.editor.saver");

    // If the user didn't setup a blog, don't continue
    if(flock.blog.service.accountCount() == 0) {
      message(msgtxt);
      return;
    }

    // Get the xhtml source (from the editor or the source pane)
    var html;
    if (($('editordeck').selectedIndex == 0) || ($('editordeck').selectedIndex == 2))
      html = gEditor.getXHtml();
    else
      html = $('editsource').value;
 
    var strbundle = document.getElementById("blogStringBundle");

    var params = {
      title: $("flock-blog-title").value,
      tags: $("flock-blog-tags").value,
      html: html,
      draftId: gDraftId
    };

    window.published = false;
    window.permalink = false;
    window.openDialog("blogMetadata.xul", "", "chrome,modal,centerscreen", params);

    if (window.published) {
      // Open a window showing the post/blog
      if (window.permalink) {
        var wm = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
        recent = wm.getMostRecentWindow("navigator:browser");

        var linkprefs = Components.classes["@mozilla.org/preferences-service;1"]
                .getService(Components.interfaces.nsIPrefService)
                .getBranch("browser.link.");
        var newWindow = linkprefs.getIntPref ("open_newwindow");

        if (newWindow == 2)
          recent.open(window.permalink);
        else {
          var browser = recent.document.getElementById('content');
          browser.loadOneTab(window.permalink, null, null, null, false);
        }
      }

      if (gRecoveryId)
        flock.blog.service.removeRecovery(gRecoveryId);

      window.controllers.removeController(gBlogEditCommandController);
      window._timer.cancel();
      window.close();
    }
    else {
      message(msgtxt);
    }
  }
}

function flock_blogEditorSetupEditorCommands() {
  if(gBlogEditCommandController) return;
  gBlogEditCommandController = new CommandController();

  gBlogEditCommandController.cmd_flock_blogNewPost = flock_blogEditorNewPostCommand;
  gBlogEditCommandController.cmd_flock_blogPublishPost = flock_blogEditorPublishPostCommand;
  gBlogEditCommandController.cmd_flock_blogSavePost = flock_blogEditorSavePostCommand;
  gBlogEditCommandController.cmd_flock_blogOpenPost = flock_blogEditorOpenPostCommand;
  gBlogEditCommandController.cmd_flock_blogClose = flock_blogClose;

  gBlogEditCommandController.cmd_flock_blogLink = flock_blogLink;
  gBlogEditCommandController.cmd_flock_blogUnlink = flock_blogUnlink;

  window.controllers.appendController(gBlogEditCommandController);
  window.addEventListener('focus',function () { window.document.commandDispatcher.updateCommands('focus'); },false);
  window.addEventListener('click',function () { window.document.commandDispatcher.updateCommands('click'); },false);
//    window.addEventListener('dragdrop',function () { alert("drop"); },false);
  window.document.commandDispatcher.updateCommands('focus');
// TO - do not call window.focus(), in XP it throws the editor window
//      into a confused state that is only resolved by re-focusing the window.
//      If you need to focus the editor use gEditor.mEditor.contentWindow.focus() instead.
// window.focus();
}

function flock_blogUpdateEditCommands() {
  flock_blogUpdateCommands($("flock_BlogEditCommands"), gBlogCommandController);
}


/*
 * Sidebar
 */

// |forceOpen| is a bool that indicates that the sidebar should be forced open.  In other words
// the toggle won't be allowed to close the sidebar.
function toggleSidebar(aCommandID, forceOpen) {

  var sidebarBox = document.getElementById("sidebar-box");
  if (!aCommandID)
    aCommandID = sidebarBox.getAttribute("sidebarcommand");

  var elt = document.getElementById(aCommandID);
  var sidebar = document.getElementById("sidebar");
  var sidebarTitle = document.getElementById("sidebar-title");
  var sidebarSplitter = document.getElementById("sidebar-splitter");

  if (!forceOpen && elt.getAttribute("checked") == "true") {
    elt.removeAttribute("checked");
    sidebarBox.setAttribute("sidebarcommand", "");
    sidebarTitle.setAttribute("value", "");
    sidebarBox.hidden = true;
    sidebarSplitter.hidden = true;
    if (aCommandID != "flock_NewsSidebarBroadcaster")
      content.focus();
    return;
  }

  var elts = document.getElementsByAttribute("group", "sidebar");
  for (var i = 0; i < elts.length; ++i)
    elts[i].removeAttribute("checked");

  elt.setAttribute("checked", "true");;

  if (sidebarBox.hidden) {
    sidebarBox.hidden = false;
    sidebarSplitter.hidden = false;
  }

  var url = elt.getAttribute("sidebarurl");
  var title = elt.getAttribute("sidebartitle");
  if (!title)
    title = elt.getAttribute("label");
  sidebar.setAttribute("src", url);
  sidebarBox.setAttribute("src", url);
  sidebarBox.setAttribute("sidebarcommand", elt.id);
  sidebarTitle.setAttribute("value", title);
  if (aCommandID != "viewWebPanelsSidebar") { // no searchbox there
    // if the sidebar we want is already constructed, focus the searchbox
    if ((aCommandID == "viewBookmarksSidebar" && sidebar.contentDocument.getElementById("bookmarksPanel"))
    || (aCommandID == "viewHistorySidebar" && sidebar.contentDocument.getElementById("history-panel")))
      sidebar.contentDocument.getElementById("search-box").focus();
    // otherwiese, attach an onload handler
    else
      sidebar.addEventListener("load", asyncFocusSearchBox, true);
  }
}

function editImageContext() {
  var node = gEditor.mNSHTMLEditor.selection.getRangeAt(0).cloneContents().firstChild;
  if (node.nodeName != "IMG")
    return;

  var params = {
    URL: node.getAttribute("src"),
    title: node.getAttribute("title"),
    alt: node.getAttribute("alt"),
    cancelled: false,
    align: ""
  };
  if (node.hasAttribute("style")) {
    if (node.getAttribute("style").match("left"))
      params.align = "left";
    else if (node.getAttribute("style").match("right"))
      params.align = "right";
    else if (node.getAttribute("style").match("center"))
      params.align = "center";
  }

  window.openDialog("chrome://browser/content/flock/blog/imageProperties.xul", "_blank", "modal,centerscreen", params);
  if (!params.cancelled) {
    var style;
    switch (params.align) {
      case "left":
        style="margin: 10px 10px 0pt 0pt; float: left;";
        break;
      case "right":
        style="margin: 0pt 0pt 10px 10px; float: right;";
        break;
      case "center":
        style="margin: 0px auto 10px; display: block; text-align: center;";
        break;
      default:
        style="";
    }

    imgnode = gEditor.mNSHTMLEditor.getSelectedElement ('img');
    imgnode.setAttribute("src", params.URL);
    imgnode.setAttribute("title", params.title);
    imgnode.setAttribute("alt", params.alt);
    imgnode.setAttribute("style", style);
  }
}

function updateContextMenuImage(event) {
  var node = gEditor.mNSHTMLEditor.selection.getRangeAt(0).cloneContents().firstChild;
  if (node)
    $("edit-image-context").hidden = (node.nodeName != "IMG");
  else
    $("edit-image-context").hidden = true;
}

/*
 * Spellchecking
 */

function updateContextMenu(event) {
  InlineSpellCheckerUI.clearSuggestionsFromMenu();
  InlineSpellCheckerUI.clearDictionaryListFromMenu();

  InlineSpellCheckerUI.init(gEditor.mNSHTMLEditor);
  InlineSpellCheckerUI.initFromEvent(document.popupRangeParent, document.popupRangeOffset);

  var canSpell = InlineSpellCheckerUI.canSpellCheck;
  var onMisspelling = InlineSpellCheckerUI.overMisspelling;
  $("spell-check-enabled").hidden = !canSpell;
  // $("spell-separator").hidden = !(canSpell || this.possibleSpellChecking);
  $("spell-separator").hidden = canSpell;
  if (canSpell)
    document.getElementById("spell-check-enabled").setAttribute("checked", InlineSpellCheckerUI.enabled);
  $("spell-add-to-dictionary").hidden = !onMisspelling;

  // suggestion list
  $("spell-suggestions-separator").hidden = !onMisspelling;
  if (onMisspelling) {
      var menu = $("blogEditorContext");
      var suggestionsSeparator = document.getElementById("spell-add-to-dictionary");
      var numsug = InlineSpellCheckerUI.addSuggestionsToMenu (menu, suggestionsSeparator , 5);
      $("spell-no-suggestions").hidden = (numsug > 0);
  } else {
      $("spell-no-suggestions").hidden = true;
  }

  // dictionary list
  $("spell-dictionaries").hidden = !InlineSpellCheckerUI.enabled;
  if (canSpell) {
    var dictMenu = $("spell-dictionaries-menu");
    var dictSep = $("spell-language-separator");
    InlineSpellCheckerUI.addDictionaryListToMenu(dictMenu, dictSep);
  }
}

function formatURL(aFormat, aIsPref) {
  var formatter = Components.classes["@mozilla.org/toolkit/URLFormatterService;1"].getService(Components.interfaces.nsIURLFormatter);

  return aIsPref ? formatter.formatURLPref(aFormat) : formatter.formatURL(aFormat);
}

function addDictionaries() {
  var uri = formatURL("browser.dictionaries.download.url", true);

  var locale = "-";
  try {
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    locale = prefService.getComplexValue("intl.accept_languages", Components.interfaces.nsIPrefLocalizedString).data;
  }
  catch (e) { }

  var version = "-";
  try {
    version = Components.classes["@mozilla.org/xre/app-info;1"]
                            .getService(Components.interfaces.nsIXULAppInfo)
                            .version;
  }
  catch (e) { }

  uri = uri.replace(/%LOCALE%/, escape(locale));
  uri = uri.replace(/%VERSION%/, version);

  var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  var newWindowPref = prefService.getIntPref("browser.link.open_newwindow");
  var where = newWindowPref == 3 ? "tab" : "window";

  openUILinkIn(uri, where);
}


/*
 * Stuff that I don't know where to put
 */

function ljuser()
{
  var strbundle = document.getElementById("editorStringBundle");
  var msgtxt = strbundle.getString("flock.blog.editor.lj.msg");

  var userid = prompt(msgtxt, "");
  var html = '<lj user="'+userid+'">';
  $('edit').contentWindow.document.execCommand("inserthtml", false, html);
}

function execCommand(cmd)
{
  $('edit').contentWindow.document.execCommand(cmd, false, null);
}

function message(aText) {
  const priority = gNotificationBox.PRIORITY_INFO_LOW;
  gNotificationBox.appendNotification(aText, "blog-autosave",
                                              "chrome://browser/skin/Info.png",
                                               priority, null);
}

function openBlogHelp() {
  window.open('http://help.flock.com/content.php?title=_Writing___Editing_Blog_Posts', 'FlockHelp', 'width=555,height=545,resizable,scrollbars');
}
