//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

function doMigrations() {
  var args = [];
  if ("arguments" in window)
    args = window.arguments;

  var oldVersion = args.shift();
  var services = args;

  var serviceLabel  = document.getElementById("migration-info");
  var msgLabel      = document.getElementById("migration-message");
  var progressMeter = document.getElementById("migration-progress");

  for (var i = 0; i < services.length; i++) {
    var service = services[i];

    var basePercent = Math.round((i / services.length) * 100);

    serviceLabel.value = service.shortname;
    msgLabel.value = "";
    progressMeter.value = basePercent;
    yield false;

    var progressWatcher = {
      divisor: services.length,
      msgLabel: msgLabel,
      progressMeter: progressMeter,
      basePercent: basePercent,
      onUpdate: function(percent, message) {
        dump("PROGRESS: " + percent + " " + message + "\n");
        this.msgLabel.value = message;
        if (percent >= 0) {
          var value = Math.round(this.basePercent + percent / this.divisor);
          this.progressMeter.value = value;
        }
      }
    };

    try {
      var classDesc = service.classDescription;
      dump("Migrate " + classDesc + "\n"); 
      var ctxt = service.startMigration(oldVersion, progressWatcher); 
      while (service.doMigrationWork(ctxt))
        yield false;
      dump("done with " + classDesc + "\n"); 
    }
    catch (e) {
      dump("Exception in migrating " + classDesc + " " + e + "\n");
    }
    finally {
      try {
        service.finishMigration(ctxt);
      }
      catch (e) { }
    }
  }
  
  // only do the following when migrating from 0.7.x (Cardinal)
  if (oldVersion.substr(0, 3) == "0.7") {
    // Append the mywolrd to the users home page if it is not already there
    var prefService = Components.classes["@mozilla.org/preferences-service;1"]
                                .getService(Components.interfaces.nsIPrefBranch);
    if (prefService) {
      if (prefService.prefHasUserValue("browser.startup.homepage") &&
          prefService.getPrefType("browser.startup.homepage") == prefService.PREF_STRING) {
        var currHomePage = prefService.getCharPref("browser.startup.homepage");
        if (!currHomePage.match(/.*about:myworld.*/)) {
          currHomePage = "about:myworld|" + currHomePage;
          prefService.setCharPref("browser.startup.homepage", currHomePage);
        }
      }
    }

    // reset toolbars during migration as a lot has changed.
    resetToolbars();
  }

  serviceLabel.value = "Done";
  msgLabel.value = "";
  progressMeter.value = 100;
  yield true;
}

// delete localstore.rdf to reset toolbars.
function resetToolbars() {
  const nsIDirectoryServiceContractID = "@mozilla.org/file/directory_service;1";
  const nsIProperties = Components.interfaces.nsIProperties;
  var directoryService =  Components.classes[nsIDirectoryServiceContractID]
                                    .getService(nsIProperties);
  var localstoreFile = directoryService.get("LStoreS", Components.interfaces.nsIFile);
  if (localstoreFile.exists())
    localstoreFile.remove(false);
}

function migrate() {
  var migrationRunner = doMigrations();

  var run = {
    notify: function(timer) {
      if (migrationRunner.next()) {
        timer.cancel();
        window.close();
      }
    }
  };

  const nsITimer = Components.interfaces.nsITimer;
  var timer = Components.classes['@mozilla.org/timer;1']
    .createInstance(nsITimer);
  timer.initWithCallback(run, 0, nsITimer.TYPE_REPEATING_SLACK);
}
