//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const nsIXMLHttpRequest             = Components.interfaces.nsIXMLHttpRequest;
const XMLHTTPREQUEST_CONTRACTID     = '@mozilla.org/xmlextras/xmlhttprequest;1';

function parseDate(datestr)
{
    var date = null;

    datestr = datestr.replace(/^\s*|\s*$/g, '');

    var dateval = Date.parse(datestr);

    if (dateval) {
        date = new Date(dateval);
    } else {
        /* this is a parser for the W3C subset of ISO 8601 dates
         * http://www.w3.org/TR/NOTE-datetime
         */
        const re_iso8601 = new RegExp(
            "^(\\d{4})(-(\\d{2,3})(-(\\d{2})" +
            "(T(\\d{2}):(\\d{2})(:(\\d{2})(\\.(\\d+))?)?" +
            "(Z|(([-+])(\\d{2}):(\\d{2})))?)?)?)?$");

        var m = datestr.match(re_iso8601);

        if (m) {
            date = new Date(Date.UTC(m[1]))

            m[3]  && date.setUTCMonth   (m[3] - 1);
            m[5]  && date.setUTCDate    (m[5]);
            m[7]  && date.setUTCHours   (m[7]);
            m[8]  && date.setUTCMinutes (m[8]);
            m[10] && date.setUTCSeconds (m[10]);

            m[12] && date.setUTCMilliseconds(('0.' + m[12]) * 1000);

            if (m[14]) {
                var tz_off = m[16] * 60 + Number(m[17]);
                tz_off *= m[15] == '-' ? -1 : 1;
                tz_off *= 60 * 1000;

                date.setTime(date.getTime() - tz_off);
            }

        }
    }

    return date;
/*
    if (date)
        return date.toString();
    else
        return null;
*/
}/*}}}*/


function jsxmlrpc_call()
{
    var listener = arguments[0];   
    var url = arguments[1];   
    var call = arguments[2];   

    var params = new Array();

    for(var i=3;i<arguments.length;++i)
    {
        params[params.length] = arguments[i];
    }
    var rpc = new JSXMLRPC();

    rpc.call(listener, url, call, params);
}

function jsxmlrpc_getXMLRPCommand(aObj) {
    //if(!aObj) return "<struct></struct>";
    if(aObj.toXMLRPCParam) return aObj.toXMLRPCParam();

    var rval = "<struct>";
    for(p in aObj)
    {
        if(p.match(/toX/)) continue;
        //var child = aObj[p].toXMLRPCParam();
        var child = jsxmlrpc_getXMLRPCommand(aObj[p]);
        rval += "<member><name>" + p + "</name><value>" + child + "</value></member>";
    }
    rval += "</struct>";
    return rval;
}

/*
Object.prototype.toXMLRPCParam = function()
{
    var rval = "<struct>";
    for(p in this)
    {
        if(p.match(/toX/)) continue;
        var child = this[p].toXMLRPCParam();
        rval += "<member><name>" + p + "</name><value>" + child + "</value></member>";
    }
    rval += "</struct>";
    return rval;
}
*/

String.prototype.toXMLRPCParam = function()
{ 
    var data = this.replace(/\]\]/g, "] ]");
    return "<string><![CDATA[" + data + "]]></string>";
}

Number.prototype.toXMLRPCParam = function()
{
    if(this == parseInt(this))
    {
        return "<int>" + this + "</int>";
    }
    else if(this == parseFloat(this))
    {
        return "<double>" + this + "</double>";
    }
    throw("Invalid Number Format");
}

Boolean.prototype.toXMLRPCParam = function()
{
    if(this==true) return "<boolean>1</boolean>";

    return "<boolean>0</boolean>";
}

Date.prototype.toXMLRPCParam = function()
{
    return "<dateTime.iso8601>" + iso8601Format(this) + "</dateTime.iso8601>";
}

Array.prototype.toXMLRPCParam = function()
{
    var rval = "<array><data>";
    for(var i=0;i<this.length;++i)
    {
        //dump(i+", jsxmlrpc_getXMLRPCommand("+this[i]+")\n");
        rval += "<value>" + jsxmlrpc_getXMLRPCommand(this[i]) + "</value>";
    }
    rval += "</data></array>";
    return rval;
}

function JSXMLRPC()
{
}

/*
 -- XML-RPC client -- : Request: <?xml version="1.0" ?><methodCall><methodName>m
 etaWeblog.getRecentPosts</methodName><params><param><value><string><![CDATA[]]><
 /string></value></param><param><value><string><![CDATA[anthony]]></string></valu
 e></param><param><value><string><![CDATA[password]]></string></value></param><pa
 ram><value><i4></i4></value></param></params></methodCall>
*/


JSXMLRPC.prototype = 
{
    INT:      1,
    BOOLEAN:  2,
    STRING:   3,
    DOUBLE:   4,
    DATETIME: 5,
    ARRAY:    6,
    STRUCT:   7,

    ERROR_HTTP:   10,
    ERROR_FAULT:   11,
    ERROR_PARSER:   12,

    _uri: null,
    _req: null,
    _listener: null,
    _error: false,

    call: function(listener, url, call, params)//arguments follow
    {
        this._listener = listener;
        var message = '<?xml version="1.0" ?>';
        message += '<methodCall><methodName>';
        message += call;
        message += '</methodName><params>';

        for(var i=0;i<params.length;++i)
        {
            var cur = params[i];
            message += "\n";
            message += '<param><value>';
            //message += cur.toXMLRPCParam();
            dump(i+", jsxmlrpc_getXMLRPCommand("+params[i]+")\n");
            message += jsxmlrpc_getXMLRPCommand(cur);
            message += '</value></param>';
            continue;
        }
        message += '</params></methodCall>';

        var inst = this;
        this._req = Components.classes[XMLHTTPREQUEST_CONTRACTID].createInstance(nsIXMLHttpRequest);
        this._req.open('POST', url, true);
        this._req.onreadystatechange = function (aEvt) 
        {
            try {
                if(inst._req.readyState == 4) 
                {
                    //FlockLogger.write("xmlrpc.txt", "\nRECEIVING\n");
                    //FlockLogger.write("xmlrpc.txt", inst._req.responseText);
                    dump("RESPONSE == " + inst._req.responseText);
                    if(inst._req.status == 200)
                    {
                        try
                        {
                            var result = inst.doParse();
                            if(inst._error)
                            {
                                listener.onFault(result);
                            }
                            else
                            {
                                dump("** result.length = "+result.length+"\n");
                                listener.onResult(result);
                                dump("** result.length = "+result.length+"\n");
                            }
                        }
                        catch(e)
                        {
                            dump(e + "\n");
                            listener.onError(inst.ERROR_PARSER);
                        }
                    }
                    else
                    {
                        listener.onError(inst._req.status);
                    }
                }
            }
            catch(e) {
               dump(e);
               listener.onError(null);
            }
        };
        //alert(message);
        //dump("\n" + message + "\n");
        //FlockLogger.write("xmlrpc.txt", "\nSENDING to " + url + "\n");
        //FlockLogger.write("xmlrpc.txt", message);
        this._req.setRequestHeader("Content-Type", "text/xml");
        this._req.send(message); 
    },

    //TODO: GET RID OF THE NEXT FUNCTION
    /*
    asyncCall: function()//arguments follow
    {
        var listener = arguments[0];   
        this._listener = listener;
        var url = arguments[1];   
        var call = arguments[2];   

        var message = '<?xml version="1.0" ?>';
        message += '<methodCall><methodName>';
        message += call;
        message += '</methodName><params>';

        for(var i=3;i<arguments.length;++i)
        {
            var cur = arguments[i];
            message += '<param><value>';
            message += cur.toXMLRPCParam();
            message += '</value></param>';
            continue;
        }
        message += '</params></methodCall>';

        var inst = this;
        this._req = new XMLHttpRequest();
        this._req.open('POST', url, true);
        this._req.send(message); 
        this._req.onreadystatechange = function (aEvt) 
        {
            if(inst._req.readyState == 4) 
            {
                if(inst._req.status == 200)
                {
                    var result = inst.doParse();
                    listener.onResult(result);
                }
                else
                {
                    listener.onError(inst.ERROR_HTTP);
                }
            }
       };
    },
    */

    doError: function(error)
    {
        this._error = error;
        //this._listener.onError(error);
    },
    doParse: function(listener, call)//arguments follow
    {
        var dom  = this._req.responseXML;
        if(!dom)
        {
            var str = this._req.responseText;
            if(!str.match(/^<\?xml/)) throw "Invalid Response From Server";
            //hackz0rplzondsf
            str = str.replace(/\r/g, "");
            str = str.replace(/\n/g, "hackz0rplzidathankyou");
            //str = str.replace(/\n/g, "");
            //dump(str);
            if(str.match(/.*?(<\?xml.+)/g))
            {
                //alert(RegExp.$1);
                var parser = new DOMParser();
                dom = parser.parseFromString(RegExp.$1,"text/xml");
                //alert(dom.firstChild.firstChild.nodeValue + " " + str + " " + dom);
            }
        }
        if(!dom) throw "Invalid Response From Server";
	    var faults = dom.getElementsByTagName("fault");

        if(faults.length>0)
        {
            var fault = faults[0];
            var data = this.extractValue(fault);
            this.doError(this.ERROR_FAULT);
            return data;
        }
	var params = dom.getElementsByTagName("param");
        if(params.length) return this.extractValue(params[0]);
    },

    extractValue: function(data)//arguments follow
    {
        var value = this.getGoodChild(data.childNodes);
        if(value.nodeName=="value")
        {
            var primitive = this.getGoodChild(value.childNodes);
            var rv;
            if(primitive)
            {
                dump("Primitive!!!\n");
                rv = this.extractPrimitive(primitive);
            }
            else
            {
                dump("NOT Primitive!!!\n");
                rv = value.firstChild.nodeValue;
            }
            return rv;
        }

        return null;
    },

    extractPrimitive: function(data)//arguments follow
    {
        if(!data) return "";
//        if(data) alert(data.nodeName + " = " + data.nodeValue);
//        if(!data) alert("ARGHH" + data);
		switch(data.nodeName)
        {
            case "struct":
                {
                    var structObj = {};
                    var struct = data.childNodes;
                    for(var i=0;i<struct.length;++i)
                    {
                        var member = struct[i];
                        if(this.isGoodChild(member))
                        {
                            var member_children = member.childNodes;
                            var name = null;
                            var value = null;
                            for(var j=0;j<member_children.length;++j)
                            {
                                var member_child = member_children[j];
                                if(this.isGoodChild(member_child))
                                {
                                    if(member_child.nodeName=="name")
                                    {
                                        name = member_child.firstChild.nodeValue;
                                    }
                                    if(member_child.nodeName=="value")
                                    {
                                        if(member_child.firstChild)
                                        {
                                            valueNode = member_child.firstChild;
                                            value = this.extractPrimitive(valueNode);
                                            if(!value)
                                            {
                                                value = member_child.firstChild.nodeValue;
                                            }
                                        }
                                        else
                                        {
                                            value = "";
                                        }
                                    }
                                    if(name && value)
                                    {
                                       structObj[name] = value; 
                                    }
                                }
                            }
                        }
                    }
                    return structObj;
                }
            break;
            case "i4":
            case "int":
                {
                    return parseInt(data.firstChild.nodeValue);
                }
            break;
            case "boolean":
            {
                    return (parseInt(data.firstChild.nodeValue)==1);
            }
            case "#text":
            {
                var rval = "";
                var child = data;
                while(child)
                {
                    rval += child.nodeValue;
                    child = child.nextSibling;
                }
                rval = rval.replace(/hackz0rplzidathankyou/g, "\n");
                return rval;
            }
            break;
            case "string":
                {
                    if(!data.firstChild)
                    {
                        return "";
                    }

                    var rval = "";
                    var child = data.firstChild;
                    while(child)
                    {
                        rval += child.nodeValue;
                        child = child.nextSibling;
                    }

                    rval = rval.replace(/hackz0rplzthankyou/g, "\n");
                    return rval;
                }
            break;
			case "dateTime.iso8601":
                {
                    var val = data.firstChild.nodeValue + "";
                    var rval = parseDate(val)
                    if(!rval) {
                        var dateutc =  Date.UTC(val.slice(0, 4), val.slice(4, 6) - 1, val.slice(6, 8), val.slice(9, 11), val.slice(12, 14), val.slice(15));
                        rval = new Date(dateutc);
                    }
                    return rval;
                    //return date;
                }
            break;
			case "array":
                {
                    dump("Got an array.\n");
                    var rv = new Array();
                    var data_tag = this.getGoodChild(data.childNodes);
                    for (var i=0;i<data_tag.childNodes.length;++i)
                    {
                        var node = data_tag.childNodes[i];
                        if(this.isGoodChild(node))
                        {
                            var primitive = this.getGoodChild(node.childNodes);
                            rv[rv.length] = this.extractPrimitive(primitive);
                        }
                    }
                    return rv;
                }
            break;
            default:
                throw new Error("parse error in xmlrpc - unknown type");
                //alert("SHIOT");
            break;
        }
        return null;
    },

    isGoodChild: function(node)
    {
        if(!node.nodeName) return false;
        if(node.nodeName.match(/text/)) return false;
        return true;
    },

    getGoodChild: function(children)
    {
        for(var i=0;i<children.length;++i)
        {
            if(this.isGoodChild(children[i])) return children[i];
        }
        return null;
    }

}

function Value()
{
}

Value.prototype = 
{
    INT:      JSXMLRPC.prototype.INT,
    BOOLEAN:  JSXMLRPC.prototype.BOOLEAN,
    STRING:   JSXMLRPC.prototype.STRING,
    DOUBLE:   JSXMLRPC.prototype.DOUBLE,
    DATETIME: JSXMLRPC.prototype.DATETIME,
    ARRAY:    JSXMLRPC.prototype.ARRAY,
    STRUCT:   JSXMLRPC.prototype.STRUCT,
    data:   "",
    toXML:   function()
    {
    },
    toXML:   function()
    {
    },
}

function iso8601Format(date) 
{
    var datetime = date.getUTCFullYear();
    var month = String(date.getUTCMonth() + 1);
    datetime += (month.length == 1 ?  '0' + month : month);
    var day = date.getUTCDate();
    datetime += (day < 10 ? '0' + day : day);

    datetime += 'T';

    var hour = date.getUTCHours();
    datetime += (hour < 10 ? '0' + hour : hour) + ':';
    var minutes = date.getUTCMinutes();
    datetime += (minutes < 10 ? '0' + minutes : minutes) + ':';
    var seconds = date.getUTCSeconds();
    datetime += (seconds < 10 ? '0' + seconds : seconds);

    return datetime;
}
