// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

// This is a library containing various functions useful for the online favorites back-end
// It is supposed to be called from the service itself, as it is accessing members
// not exposed in the interface (the coop instance, the logger...)

const DEFAULT_REFRESH_INTERVAL = 5*60; // update any bm stream every 5 minutes

Components.classes['@mozilla.org/moz/jssubscript-loader;1']
    .getService(Components.interfaces.mozIJSSubScriptLoader)
    .loadSubScript("chrome://browser/content/flock/common/flocksafe.js");


function stringify (foo) {
  if (foo==null) {
    return '';
  } else {
    return foo;
  }
}

function dateify (foo) {
  if (foo == null) {
    return 0;
  } else {
    return foo.getTime ();
  }
}

function sanitize_tags (tags) {
  return tags?tags.split(/[\s,]/).sort().join(','):'';
}

function bookmarks_match (bm1, bm2) {
  return (bm1.private == bm2.private && /* booleans - nice and easy */
          stringify (bm1.name) == stringify (bm2.name) &&
          stringify (bm1.description) == stringify (bm2.description) &&
          sanitize_tags(bm1.tags) == sanitize_tags(bm2.tags) &&
          dateify (bm1.datevalue) == dateify (bm2.datevalue));
}

var onlineFavoritesBackend = {
  getCompTK: function () {
    if (!gCompTK) {
      gCompTK = Cc["@flock.com/singleton;1"].getService(Ci.flockISingleton)
        .getSingleton("chrome://browser/content/flock/services/common/load-compTK.js")
        .wrappedJSObject;
    }
    return gCompTK;
  },

  createAccount: function (aService, aAccountID, aIsTransient) {
    var account_urn = aService.urn + ":" + aAccountID;
    var account;
    if (!aService._coop.Account.exists (account_urn)) {
      account = new aService._coop.Account (account_urn, {
        name: aAccountID,
        serviceId: CONTRACT_ID,
        service: aService._service,
        accountId: aAccountID,
        favicon: aService._ctk.favicon,
        URL: aService.getUserUrl(aAccountID),
        isTransient: aIsTransient,
        showInSidebar: false
      });
      aService._coop.accounts_root.children.addOnce (account);
    } else {
      account = aService._coop.get (account_urn);
    }

    // Create a "stream" for the online bookmarks
    var bookmarks_urn = account_urn+":bookmarks";
    if (!aService._coop.OnlineBookmarksStream.exists(bookmarks_urn)) {
      var bookmarks = new aService._coop.OnlineBookmarksStream(bookmarks_urn, {
        name: aAccountID+ ' on '+aService.title,
        userid: aAccountID,
        serviceId: aService._ctk.contractID,
        favicon: aService._ctk.favicon,
        isTransient: aIsTransient,
        refreshInterval: DEFAULT_REFRESH_INTERVAL
      });
      aService._coop.onlinebookmarks_root.children.addOnce(bookmarks);
    }
    
    // Create the "no tag" folder
    var noTagFolder = new aService._coop.Folder(bookmarks.id()+":notag",
      {name: flockGetString("favorites/favorites", "flock.favs.online.noTag")});
    bookmarks.children.add(noTagFolder);
    
    var acct = aService.getAccount(account_urn);
    return acct;
  },

  removeAccount: function (aService, aAccountUrn) {
    var acctCoopObj = aService._coop.get(aAccountUrn);
    // Next delete the bookmarks that are solely associated with this account
    var bookmarks_urn = aAccountUrn+':bookmarks';
    var bmStream = aService._coop.get(bookmarks_urn);
    if (bmStream) {
      // Remove the Delicious Bookmarks node from its parents
      var parents = bmStream.getParents();
      for (var i = 0; i < parents.length; i++) {
        parents[i].children.remove(bmStream);
      }
    } else {
      // this._logger.debug("  DID NOT find bookmark stream "+bookmarks_urn); 
    }
    // Next remove the account from any parents it might have
    var parents = acctCoopObj.getParents();
    for (var i = 0; i < parents.length; i++) {
      // this._logger.debug("   removing from parent "+i);
      parents[i].children.remove(acctCoopObj);
    }
    // kill the account node
    acctCoopObj.destroy();

    // Finally, slowly delete the bookmarks
    var synchronize = function removeAccount_synchronize (aShouldYield) {
      var i = 0;
      var bmChildren = bmStream.children.enumerate();
      while (bmChildren.hasMoreElements()) {
        var tag = bmChildren.getNext();
        var myEnum = tag.children.enumerate();
        while (myEnum.hasMoreElements()) {
          var bm = myEnum.getNext();
          var parents = bm.getParents();
          for (var j = 0; j < parents.length; j++) {
            parents[j].children.remove(bm);
          }
          bm.destroy();
          if (i++ % 20 == 0 && aShouldYield()) {
            yield;
          }
        }
      }
      bmStream.destroy();
    }

    if (bmStream) {
      getCompTK().schedule(gTimers, 0.4, 40, synchronize);
    }
  },

  updateBookmark: function (aService, aAccountUrn, aServerBookmark, aPrivate) {
    var bookmark_urn = aAccountUrn+":"+aServerBookmark.URL;
    var localBookmark = null;
    var tagList;

    // Create/update the bookmark itself
    if (aService._coop.Bookmark.exists (bookmark_urn)) {
      // Need to check if any modification is needed before we assert anything
      var localBookmark = aService._coop.get (bookmark_urn);
      if (bookmarks_match(aServerBookmark, localBookmark)) {
        aService._logger.debug(aServerBookmark.URL+" is unchanged. Do nothing.");
        return;
      }
      aService._logger.debug(aServerBookmark.URL+" has been modified on the server. Updating.");
      if (stringify (aServerBookmark.name) != stringify (localBookmark.name))
        localBookmark.name = aServerBookmark.name;
      if (stringify (aServerBookmark.description) != stringify (localBookmark.description))
        localBookmark.description = aServerBookmark.description;
      if (sanitize_tags(aServerBookmark.tags) != sanitize_tags(localBookmark.tags)) {
        localBookmark.tags = sanitize_tags(aServerBookmark.tags)
        var tagenum = localBookmark.tag.enumerate();
        while (tagenum.hasMoreElements()) {
          var tag = tagenum.getNext();
          localBookmark.tag.remove(tag);
        }
      }
      var date  =  new Date();
      localBookmark.LastModifiedDate = date;
      localBookmark.private = aPrivate;
      localBookmark.save();
    }
    else {
      // New bookmark
      aService._logger.debug(aServerBookmark.URL+" is a new bookmark. Creating.");
      localBookmark = new aService._coop.Bookmark (bookmark_urn, {
        URL: aServerBookmark.URL,
        name: aServerBookmark.name,
        description: aServerBookmark.description,
        tags: aServerBookmark.tags,
        datevalue: aServerBookmark.datevalue,
        hash: aServerBookmark.hash,
        flockType: "bookmark",
        private: aServerBookmark.shared=='no'?true:false,
        isPollable: false
      });
      // bookmarks.children.add(localBookmark);
    }
    var tagList = aServerBookmark.tags.split(/[\s,]/);
    for (i = 0; i < tagList.length; i++) {
      localBookmark.tag.addOnce(tagList[i]);
    }
    var bookmarks = aService._coop.get(aAccountUrn + ":bookmarks");
    onlineFavoritesBackend.classifyBookmark (aService, bookmarks, localBookmark, tagList);
  },

  classifyBookmark: function (aService, aBookmarks, aLocalBookmark, aTagList) {
    // Delete the bookmark from the tag folders, and delete empty tags
    var parents = aLocalBookmark.getParents();
    while (parents.length > 0) {
      var parent_ = parents.pop();
      parent_.children.remove(aLocalBookmark);
      if (parent_.children.count() == 0) {
        aBookmarks.children.remove(parent_);
        parent_.destroy();
      }
    }

    var tagFolder;
    // Add the bookmark to the tag folders it should be
    if ((aTagList.length == 1 && aTagList[0].length ==0) || (aTagList.length == 0) || (aTagList == [""])) {
      var notagUrn = aBookmarks.id()+":notag";
      tagFolder = aService._coop.get(notagUrn);
      tagFolder.children.add(aLocalBookmark);
    }
    else for (i = 0; i < aTagList.length; i++) {
      var tagUrn = aBookmarks.id() + ":tag:" + aTagList[i];
      if (aService._coop.Folder.exists(tagUrn)) {
        tagFolder = aService._coop.get(tagUrn);
      }
      else {
        tagFolder = this.createTag(aService, aBookmarks, aTagList[i], tagUrn);
      }
      tagFolder.children.add(aLocalBookmark);
    }
  },

  createTag: function (aService, aBookmarks, aTag, aTagUrn) {
    var tagFolder = new aService._coop.Folder(aTagUrn, 
      { name: aTag }
    );
    var myEnum = aBookmarks.children.enumerate();
    var found = false;
    var i = 1;
    myEnum.getNext(); // Ignore "notag"
    while (myEnum.hasMoreElements() && !found) {
      var currentTag = myEnum.getNext();
      if (aTag < currentTag.name) {
        found = true;
        aBookmarks.children.insertAt(tagFolder, i+1);
      }
      i++;
    }
    if (!found)
      aBookmarks.children.add(tagFolder);
    return tagFolder;
  },

  destroyBookmark: function (aService, aAccountId, aUrl) {
    var bookmark_urn = aService.urn+":"+aAccountId+":"+aUrl;

    if (aService._coop.Bookmark.exists (bookmark_urn)) {
      var bookmarks = aService._coop.get(aService.urn+":"+aAccountId+":bookmarks");
      var bookmark = aService._coop.get(bookmark_urn);

      bookmarks.children.remove(bookmark);
      bookmark.destroy();
    }
  },

  updateLocal: function (aService, aPostsList, aAccountUrn) {
    // Update the last update time for the bookmark stream
    var localBookmarks = aService._coop.get(aAccountUrn+':bookmarks');
    localBookmarks.last_update_time = (new Date());

    var synchronize = function updateLocal_synchronize (aShouldYield) {
      var urlHash = {}; // Useful to optimize deletion

      // Create new bookmarks, or update existing bookmarks
      for (var i=0; i < aPostsList.length; i++) {
        if (aPostsList[i].tags == "system:unfiled")
          aPostsList[i].tags = "";
        // To optimize deletion: put the URLs in a hash
        urlHash[aPostsList[i].URL] = true;
        onlineFavoritesBackend.updateBookmark (aService, aAccountUrn, aPostsList[i]);
        if (i % 10 == 0 && aShouldYield())
          yield;
      }

      // Remove locally bookmarks deleted on the server
      var localLength = 0;

      var i = 0;
      var _enum = localBookmarks.children.enumerate();
      while (_enum.hasMoreElements()) {
        var localTag = _enum.getNext();
        var _enum2 = localTag.children.enumerate();
        while (_enum2.hasMoreElements()) {
          var localBookmark = _enum2.getNext();
          if (!urlHash[localBookmark.URL]) {
            aService._logger.debug(localBookmark.URL+" has been deleted on the server. Deleting locally.");
            var parents = localBookmark.getParents();
            for (var j = 0; j < parents.length; j++) {
              parents[j].children.remove(localBookmark);
            }
            localBookmark.destroy();
          }
        }
        if (++i % 5 == 0 && aShouldYield())
          yield;
      }
      
      // Remove empty tags
      var _enum = localBookmarks.children.enumerate();
      while (_enum.hasMoreElements()) {
        var tag = _enum.getNext();
        var _enum2 = tag.children.enumerate();
        if (!_enum2.hasMoreElements() &&
            (tag.id() != localBookmarks.id() + ":notag")) {
          // The tag is empty, let's delete it
          // (don't delete "No Tag")
          localBookmarks.children.remove(tag);
          tag.destroy();
        }
      }
    }
    
    getCompTK().schedule(gTimers, 0.4, 40, synchronize);
  },

  updateTags: function (aService, aAccountId, aTags) {
    var localBookmarks = aService._coop.get(aService.urn+':'+aAccountId+':bookmarks');
    tagenum = localBookmarks.tag.enumerate();
    // Remove the existing tags
    while (tagenum.hasMoreElements())
      localBookmarks.tag.remove(tagenum.getNext());
    // Put the new tags
    for (var i=0; i<aTags.length; i++)
      localBookmarks.tag.addOnce(aTags[i]);
  },

  showNotification: function (aMessage) {
    var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                     .getService(Components.interfaces.nsIWindowMediator);
    var topNavWindow = wm.getMostRecentWindow("navigator:browser");
    
    var notificationBox = topNavWindow.gBrowser.getNotificationBox();
    var notification = notificationBox.getNotificationWithValue("favorite-error");
    if (notification) {
      notification.label = aMessage;
    }
    else {
      notificationBox.appendNotification(aMessage,
        "favorite-error",
        "chrome://browser/skin/Info.png",
        notificationBox.FLOCK_PRIORITY_HIGH, null);
    }
  }

}
