//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

var feedManager;
var subscriptionTree;
var contentBrowser;
var promptService;
var dragService;
var scrollTimer;
var windowManager;

var faves = Components.classes['@mozilla.org/rdf/datasource;1?name=flock-favorites'].getService(Components.interfaces.nsIRDFDataSource);
var faves_coop = Components.classes['@flock.com/singleton;1'].getService(Components.interfaces.flockISingleton).getSingleton('chrome://browser/content/flock/common/load-faves-coop.js').wrappedJSObject;

var newsDNDObserver = {

  onDragOver: function(aEvent, aFlavor, aSession) {
    dump("newsDNDObserver: onDragOver\n");
  },
    
  onDragEnter: function(aEvent, aSession) {
    dump("newsDNDObserver: onDragEnter\n");
  },
    
  onDragExit: function(aEvent, aSession) {
    dump("newsDNDObserver: onDragExit\n");
  },

  onDragStart: function(event, transferData, action) {
    var index = subscriptionTree.view.selection.currentIndex;
    if(index != -1) {
      var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
      var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
      var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
    } else {
      return;
    }
    transferData.data = new TransferData();
    transferData.data.addDataForFlavour("moz/rdfitem", id);
  },
  
  onDrop: function(aEvent, aDropData, aSession) {
    dump("newsDNDObserver: onDrop: contentType: " + aDropData.flavour.contentType + " data: " + aDropData.data + "\n");
    switch(aDropData.flavour.contentType) {
      case "application/x-moz-file":
        var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
        var fileHandler = ios.getProtocolHandler("file").QueryInterface(Components.interfaces.nsIFileProtocolHandler);
        var url = fileHandler.getURLSpecFromFile(aDropData.data);
        break;
      case "text/x-moz-url":
        var url = aDropData.data.split("\n")[0];
        break;
    }
    //var feedService = Components.classes["@flock.com/feed-service;1"].getService(Components.interfaces.flockIFeedService);
    //feedService.subscribeFeed(url, null, null, false);
  },
  
  getSupportedFlavours: function() {
    var flavors = new FlavourSet();
    flavors.appendFlavour("application/x-moz-file", "nsIFile");
    flavors.appendFlavour("text/x-moz-url");
    return flavors;
  }
  
}

var feedsContainerObserver = {

  canDrop: function(aDOMNode, aOrientation) {
    // Can drop on any node, any orientation
    return true;
  },
  
  onDrop: function(aDOMNode, aOrientation, aDropData) {
    var getFeedParent = function (aCoopFeed) {
      var parents = aCoopFeed.getParents();
      for (var i = 0; i < parents.length; i++) {
        if (parents[i].isA(faves_coop.FeedFolder) ||
            parents[i].id() == "urn:flock:feedcontext:news")
        {
          return parents[i];
        }
      }
      return null;
    };

    var coopFeed = faves_coop.get(aDropData.data);

    var newsContext = feedManager.getFeedContext('news');
    var url = aDropData.data.split("\n")[0];
    var ios = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
    var uri = ios.newURI(url, null, null);
    
    // Calculate insert folder and position
    var pos;
    var target = faves_coop.get(aDOMNode.id);
    var newParent;
    if (target.isA(faves_coop.FeedFolder) && aOrientation == "on") {
      newParent = target;
      pos = -1;
    } else {
      newParent = getFeedParent(target);
      pos = newParent.children.indexOf(target);
      if (aOrientation == "after") {
        pos++;
      }
    }

    if (!coopFeed) { // Add new feed
      var feedFolder = feedManager.getFeedFolderItem(newParent.id());
      feedManager.getFeed(uri, {
        onGetFeedComplete: function (aFeed) {
          feedFolder.subscribeFeed(aFeed);
        }
      });
    } else { // Move an existing feed
      // If someone drops a folder on itself...
      if (coopFeed.id() == newParent.id() || isAncestor(coopFeed, newParent)) {
        return;
      }

      // Remove from the original parent
      var parent = getFeedParent(coopFeed);
      parent.children.remove(coopFeed);

      // Add to the new parent
      if (pos < 0) {
        newParent.children.add(coopFeed);
      } else {
        newParent.children.insertAt(coopFeed, pos++);
      }
    }
  },
  
  onDragStart: function(aDOMNode) {
  }

}

var treeBuilderObserver = {
  canDrop: function(index, orient) {
    dump("treeBuilderObserver: canDrop: index: " + index + " orient: " + orient + "\n");
    if (index == -1) return false;
    var dragSession = dragService.getCurrentSession();
    var target_type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
    if (dragSession.isDataFlavorSupported("moz/rdfitem")) {
      var source_type = subscriptionTree.view.getCellText(subscriptionTree.view.selection.currentIndex, subscriptionTree.columns.getNamedColumn("type"));
      if (source_type == "root" || target_type == "root" || target_type == "flagged_collection") return false;
      if (source_type == "collection" && orient == 0) return false;
      return true;
    }
    if (dragSession.isDataFlavorSupported("text/x-moz-url")) {
      if (target_type == "root" || target_type == "flagged_collection") return false;
      return true;
    }
    return false;
  },
  
  onDrop: function(index, orient) {
    var dragSession = dragService.getCurrentSession();

    if (dragSession.isDataFlavorSupported("moz/rdfitem")) {
      var trans = nsTransferable.createTransferable();
      trans.addDataFlavor("moz/rdfitem");
      dragSession.getData(trans, 0);
      var data = { };
      var length = { };
      trans.getTransferData("moz/rdfitem", data, length);
      data.value.QueryInterface(Components.interfaces.nsISupportsString);
      //var source = data.value.toString();
  
      var source = subscriptionTree.builder.getResourceAtIndex(subscriptionTree.view.selection.currentIndex).ValueUTF8;
      var sourceType = subscriptionTree.view.getCellText(subscriptionTree.view.selection.currentIndex, subscriptionTree.columns.getNamedColumn("type"));
      var target = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
      
      var isContainerTarget = subscriptionTree.view.isContainer(index);
      var isContainerTargetExpanded = subscriptionTree.view.isContainerOpen(index);
      var isContainerTargetEmpty = subscriptionTree.view.isContainerEmpty(index);
      
      if (isContainerTarget && isContainerTargetExpanded && !isContainerTargetEmpty && orient == 1) {
        target = subscriptionTree.builder.getResourceAtIndex(index + 1).ValueUTF8;
        orient = -1;
      }
      
      switch (sourceType) {
        case "feed":
          //feedService.moveFeed(source, target, orient);
          break;
        case "collection":
          //feedService.moveCollection(source, target, orient);
          break;
      }
      
      dump("treeBuilderObserver: onDrop: source: " + source + " target: " + target + " sourceType: " + sourceType + " orient: " + orient + "\n");
    }
    
    if (dragSession.isDataFlavorSupported("text/x-moz-url")) {
      var trans = nsTransferable.createTransferable();
      trans.addDataFlavor("text/x-moz-url");
      dragSession.getData(trans, 0);
      var data = { };
      var length = { };
      trans.getTransferData("text/x-moz-url", data, length);
      data.value.QueryInterface(Components.interfaces.nsISupportsString);
      var source = data.value.toString().split("\n")[0];
      
      var target = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
      
      var isContainerTarget = subscriptionTree.view.isContainer(index);
      var isContainerTargetExpanded = subscriptionTree.view.isContainerOpen(index);
      var isContainerTargetEmpty = subscriptionTree.view.isContainerEmpty(index);
      
      if (isContainerTarget && isContainerTargetExpanded && !isContainerTargetEmpty && orient == 1) {
        target = subscriptionTree.builder.getResourceAtIndex(index + 1).ValueUTF8;
        orient = -1;
      }
      
      //feedService.subscribeFeed(source, target, orient, false);
      
      dump("treeBuilderObserver: onDrop: source: " + source + " target: " + target + " isContainerTarget: " + isContainerTarget + " isContainerTargetExpanded: " + isContainerTargetExpanded + " isContainerTargetEmpty: " + isContainerTargetEmpty + "\n");
    }
  },
  
  onToggleOpenState: function() {
  }
};

var dropListener = {
  handleEvent: function(event) {
    var dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
    var dragSession = dragService.getCurrentSession();
    var transferable = Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
    transferable.addDataFlavor("text/unicode");
    dragSession.getData(transferable, 0);
    var flavor = {};
    var data = {};
    var length = {};
    transferable.getAnyTransferData(flavor, data, length);
    
    var subscribeURL = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIStandardURL);
    subscribeURL.init(Components.interfaces.nsIStandardURL.URLTYPE_STANDARD, -1, data.value.QueryInterface(Components.interfaces.nsISupportsString).data.split("\n")[0], null, null);
    subscribeURL.QueryInterface(Components.interfaces.nsIURL);
    
    var feedManagerListener = {
      onGetFeedComplete: function(feed) {
        if (event.target.localName == "richtree") {
          feedManager.getFeedContext("news").getRoot().subscribeFeed(feed);
          return;
        }
        var targetItem = feedManager.getFeedFolderItem(event.target.id);
        var insertPos = event.target.getAttribute("insertPos");
        var parentFolder = targetItem.getFolder();
        if (targetItem instanceof Components.interfaces.flockIFeed) {
          parentFolder.subscribeFeedWithPosition(feed, targetItem, parentFolder.ORIENT_BELOW);
        }
        if (targetItem instanceof Components.interfaces.flockIFeedFolder) {
          var orientation;
          switch (insertPos) {
            case "before": orientation = parentFolder.ORIENT_ABOVE; break;
            case "on": orientation = parentFolder.ORIENT_INSIDE; break;
            case "after": orientation = parentFolder.ORIENT_BELOW; break;
          }
          parentFolder.subscribeFeedWithPosition(feed, targetItem, orientation);
        }
      },
      onError: function(error) {
      }
    }
    
    feedManager.getFeed(subscribeURL, feedManagerListener);
  }
}

function init() {
  var metrics = Components.classes['@flock.com/metrics-service;1'].getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('news open');

  feedManager = Components.classes["@flock.com/feed-manager;1"].getService(Components.interfaces.flockIFeedManager);
  
  subscriptionTree = document.getElementById("feedsContainer");
  
  subscriptionTree.addEventListener("richtreedrop", dropListener, true);
  subscriptionTree.addObserver(feedsContainerObserver);
  
  promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);
  dragService = Components.classes["@mozilla.org/widget/dragservice;1"].getService(Components.interfaces.nsIDragService);
  scrollTimer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
  windowManager = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
  
  var recentWindow = windowManager.getMostRecentWindow("navigator:browser");
  contentBrowser = recentWindow.document.getElementById('content');
}

function updateSelection() {
  if(contentBrowser.currentURI.scheme == "feed" || contentBrowser.currentURI.scheme == "feeds") {
    var feedUrl = decodeURIComponent(contentBrowser.currentURI.spec.split(contentBrowser.currentURI.scheme + ":")[1]);
    dump("feedUrl: " + feedUrl + "\n");
    for (var c = 0; c < subscriptionTree.view.rowCount; c++) {
      switch (subscriptionTree.view.getCellText(c, subscriptionTree.columns.getNamedColumn("type"))) {
        case "feed":
          if (subscriptionTree.builder.getResourceAtIndex(c).ValueUTF8 == feedUrl) {
            subscriptionTree.view.selection.select(c);
          }
          break;
        case "root":
          if (feedUrl == "") {
            subscriptionTree.view.selection.select(c);
          }
          break;
        case "collection":
          if (feedUrl == subscriptionTree.view.getCellText(c, subscriptionTree.columns.getNamedColumn("title"))) {
            subscriptionTree.view.selection.select(c);
          }
          break;
        case "flagged_collection":
          if (feedUrl == "saved") {
            subscriptionTree.view.selection.select(c);
          }
          break;
      }
    }
  }
}

function close() {
  var metrics = Components.classes['@flock.com/metrics-service;1'].getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('news close');
}

function doOnClick(event) {
  dump("News: onClick\n");
  var obj = {};
  var row = {};
  subscriptionTree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, {}, obj);
  row = row.value;
  if(event.button == 2 || event.originalTarget.localName != "treechildren" || row == -1) {
    return;
  }
  if(event.ctrlKey || event.metaKey) {
    var newTab = contentBrowser.addTab();
    contentBrowser.selectedTab = newTab;
  }
  if(obj.value == "twisty") return;
  renderContent();
}

function doOnKeyPress(event) {
  dump("News: onKeyPress: " + event.keyCode + "\n");
  if (event.keyCode == 38 || event.keyCode == 40) {
    var timerCallback = {
      notify: function(aTimer) { renderContent(); }
    }
    scrollTimer.cancel();
    scrollTimer.initWithCallback(timerCallback, 400, scrollTimer.TYPE_ONE_SHOT);
  }
}

function renderContent() {
  var index = subscriptionTree.view.selection.currentIndex;
  if(index != -1) {
    var id = subscriptionTree.builder.getResourceAtIndex(index).ValueUTF8;
    var title = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("title"));
    var type = subscriptionTree.view.getCellText(index, subscriptionTree.columns.getNamedColumn("type"));
  } else {
    return;
  }
  
  dump("News: index: " + index + " type: " + type + " id: " + id + "\n");
  
  switch (type) {
    case "feed":
      contentBrowser.loadURI("feeds:" + encodeURIComponent(id));
      break;
    case "root":
      contentBrowser.loadURI("feeds:");
      break;
    case "collection":
      contentBrowser.loadURI("feeds:" + encodeURIComponent(title));
      break;
    case "flagged_collection":
      contentBrowser.loadURI("feeds:saved");
      break;
  }
  
  var prefs = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefBranch);
  if (prefs.getBoolPref("flock.feeds.auto_mark_as_read", true)) {
    if (type != "root") {
      function latentMark() {
        //feedService.markAllRead(id);
        subscriptionTree.view.selection.select(index);
      }
      setTimeout(latentMark, 500);
    }
  }
}

function newFolder(event) {
  var check = {value: false};
  var input = {value: ""};
  var strBundle = document.getElementById('stringBundle');
  var result = promptService.prompt(null,
    strBundle.getString("flock.feed.newCollection.title"),
    strBundle.getString("flock.feed.newCollection.msg"), input, null, check)
  if (result) {
    feedManager.getFeedContext("news").getRoot().addFolder(input.value);
  }
}

function editItem(event) {
  var urn = subscriptionTree.selectedItem.id;
  var item = feedManager.getFeedFolderItem(urn);
  
  var check = {value: false};
  var input = {value: item.getTitle()};
  var strBundle = document.getElementById('stringBundle');
  if (item instanceof Components.interfaces.flockIFeed) {
    var result = promptService.prompt(null, strBundle.getString("flock.feed.editFeed.title"), strBundle.getString("flock.feed.editFeed.msg"), input, null, check);
    if (result) {
      item.setTitle(input.value);
    }
  }
  if (item instanceof Components.interfaces.flockIFeedFolder) {
    var result = promptService.prompt(null, strBundle.getString("flock.feed.editSection.title"), strBundle.getString("flock.feed.editSection.title"), input, null, check);
    if (result) {
      item.setTitle(input.value);
    }
  }
}

function deleteItem(event) {
  var urn = subscriptionTree.selectedItem.id;
  var item = feedManager.getFeedFolderItem(urn);
  if (item instanceof Components.interfaces.flockIFeed) {
    item.getFolder().unsubscribeFeed(item);
  }
  if (item instanceof Components.interfaces.flockIFeedFolder) {
    item.getFolder().removeFolder(item);
  }
}

function markAsRead() {
  var urn = subscriptionTree.selectedItem.id;
  feedManager.getFeedFolderItem(urn).markRead();
}

function importSourcePack() {
  window.openDialog("chrome://browser/content/flock/feeds/importOPML.xul", "ImportOPML", "modal,centerscreen");
}

function manuallyAddSource() {
  window.openDialog("chrome://browser/content/flock/feeds/addFeedUrl.xul", "AddFeedUrl", "modal,centerscreen");
}

function onClickItem(element) {
  if(contentBrowser.currentURI.spec == "about:myworld") {
    var tabAdded = contentBrowser.loadOneTab("flock://favorites/" + encodeURIComponent(element.getAttribute("id")), null, null, null, null, false);
    contentBrowser.selectedTab = tabAdded;
  } else {
    contentBrowser.loadURI("flock://favorites/" + encodeURIComponent(element.getAttribute("id")));
  }
}

function isAncestor(aItem1, aItem2) {
  var parents = aItem2.getParents();
  for (var i in parents) {
    if ((parents[i].id() == aItem1.id()) || isAncestor(aItem1, parents[i])) {
      return true;
    }
  }
  return false;
}
