//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

/* photo magic launches uploader when a file/photo is dropped onto a page
 */

var photoMagicDND = {
  canHandleMultipleItems: true,
  isAPhoto: function (aURL) {
    if (aURL.indexOf("file") != 0) return false;
    var lc = aURL.toLowerCase();
    if (lc.match(/.+\.jpe?g$/)) return true;
    if (lc.match(/.+\.png$/)) return true;
    if (lc.match(/.+\.gif$/)) return true;
    if (lc.match(/.+\.bmp$/)) return true;
    return false;
  },
  onDrop: function (aEvent, aXferData, aDragSession) {
    var xferData = aXferData.first.first;

    // bail if they aren't dropping in a textarea (what about html editor/content-editable? -- ja)
    if (aEvent.target.localName.toLowerCase() != 'textarea') {
      // XXX: TODO
      // This needs to be put into it's own overlay and own js file for proper design and processing.
      // Special Case: Piczo.  If dropped from mediabar to piczo edit page, pass to piczo service 
      // for handling.
      var url = transferUtils.retrieveURLFromData(xferData.data, xferData.flavour.contentType);
      if(url) {
        var apiMgr = Components.classes["@flock.com/photo-api-manager;1?"].getService(Components.interfaces.flockIPhotoAPIManager);
        var svcs = apiMgr.services;
        while (svcs.hasMoreElements()) {
          var svc = svcs.getNext();

          if (svc.shortName == "piczo") {
            // If logged in and current page is 
            if (svc.isLoggedIn) {
              svc.QueryInterface(Components.interfaces.flockIPiczoService);
              if (svc.handleDragDrop(url, aEvent.clientX, aEvent.clientY)) {
                return;
              } else { 
              }
            }
            break;
          }
        }
      }
      
      contentAreaDNDObserver.onDrop(aEvent, xferData, aDragSession);
      return;
    }
    this.textareaTarget = aEvent.target;

    // bail if they're dropping a selection of images (we just handle one)
    if (aXferData.dataList.length > 1) {

      var localeService = Components.classes["@mozilla.org/intl/nslocaleservice;1"].getService(Components.interfaces.nsILocaleService);
      var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
      var stringBundle = stringBundleService.createBundle("chrome://flock/locale/photo/photo.properties", localeService.getApplicationLocale());

      alert(stringBundle.GetStringFromName("flock.photo.magic.drag"));
      return;
    }
    
    // If there is no account, offer to configure one now
    if (!flock_photouploader.service.hasOnePhotoAccount()) {
      flock_photouploader.service.promptForAccount();
      return;
    }
    
    var url = transferUtils.retrieveURLFromData(xferData.data, xferData.flavour.contentType);

    // valid urls don't contain spaces ' '; if we have a space it
    // isn't a valid url, or if it's a javascript: or data: url,
    // bail out
    if (!url || !url.length || url.indexOf(" ", 0) != -1 ||
        /^\s*(javascript|data):/.test(url))
      return;

    getBrowser().dragDropSecurityCheck(aEvent, aDragSession, url);

    if (this.isAPhoto(url)) {
      var inst = this;
      var photoUploadListener = {
        onMediaImportStart: function( aFilename, aCount) {
        },
        onMediaImportFinish: function( aUpload, aCount ) {
          flock_photo.util.launchSinglePhotoUpload(aUpload, inst.textareaTarget);
          inst.textareaTarget = null;
        },
        onMediaImportError: function(aFilename) {
          alert('The photo "' + aFilename + '" could not be uploaded because it is invalid.');
        }
      }
      Components.classes['@flock.com/metrics-service;1'].getService(Components.interfaces.flockIMetricsService).reportCount('uploader from photoMagic');
      flock_photouploader.service.importSingleMediaItem(photoUploadListener, url);
      aEvent.preventBubble();
    } else {
//@line 115 "/home/build/tinderbuild_blue/src/flock/mozilla/browser/components/flock/photo/content/photoMagic.js"
      var textarea = aEvent.target.QueryInterface(Components.interfaces.nsIDOMNSEditableElement);
      textarea.editor.insertFromDrop(aEvent);
//@line 123 "/home/build/tinderbuild_blue/src/flock/mozilla/browser/components/flock/photo/content/photoMagic.js"
    }
  },

  getSupportedFlavours: function() {
    return contentAreaDNDObserver.getSupportedFlavours();
  }
};
