/*
 * BEGIN FLOCK GPL
 * 
 * Copyright Flock Inc. 2005-2007
 * http://flock.com
 * 
 * This file may be used under the terms of of the
 * GNU General Public License Version 2 or later (the "GPL"),
 * http://www.gnu.org/licenses/gpl.html
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 * 
 * END FLOCK GPL
 */

const KEEPACCOUNT_TIMER = 5000;
const SERVICE_SHOW_PREF_BRANCH = "flock.notification.show.accounts";
const SERVICE_SHOW_PREF_HEADER = ".account-setup-info-";

var accountsSidebar =  {
  gKeepAccountTimer: null,
  gAccountsArea: new Array("media", "bookmarks", "manualBlog", "disabled"), 
  services: null,
  prefService:Components.classes["@mozilla.org/preferences-service;1"]
                              .getService(Components.interfaces.nsIPrefBranch),
  prefBranch:Components.classes["@mozilla.org/preferences-service;1"]
                               .getService(Components.interfaces.nsIPrefService)
                               .getBranch(SERVICE_SHOW_PREF_BRANCH),                             
  /*
   Observer for the AccountsRoot rdf
   */
  onAssert            : function(ds, source, predicate, target){
    if (predicate.Value == "http://flock.com/rdf#isTransient" ||
        predicate.Value == "http://flock.com/rdf#authtoken") {
      $("activeAccountsList").builder.rebuild();
    }
  },
  onUnassert          : function(ds, source, predicate, target){
    if (source.Value == "http://flock.com/rdf#AccountsRoot") {
      $("activeAccountsList").builder.rebuild();
    }
  },
  onChange            : function(ds, source, predicate, oldTarget, newTarget){
    if ((predicate.Value == "http://flock.com/rdf#isAuthenticated") ||
        (predicate.Value == "http://flock.com/rdf#isTransient") ) {
      $("activeAccountsList").builder.rebuild();
    }
  },
  onMove              : function(ds, oldSource, newSource, predicate, target){},
  onBeginUpdateBatch  : function(ds){},
  onEndUpdateBatch    : function(ds){},

  /*
   Preference observer for the "flock.notification.show." branch
   */

  observe             : function(subject, topic, data) {
    // Observe commands to select a service when opening
    if (topic == "accountsSidebar") {
      var availableServiceList = $("availableServicesList");
      var folderList = availableServiceList.children;
      for (var iFolderCount = 0; iFolderCount < folderList.length; iFolderCount++) {
        var foundItem = false;
        var mFolder = document.getAnonymousElementByAttribute(folderList[iFolderCount], "anonid", "title");
        if (mFolder.value == data) {
          // They just want the folder open with no particular service selected.
          foundItem = true;
        } else {
          // Check if they want a service in this groupsvn 
          var serviceList = folderList[iFolderCount].children;
          for (var iServiceCount = 0; iServiceCount < serviceList.length && !foundItem; iServiceCount++) {
            if (serviceList[iServiceCount].value == data) {
              // We found a matching service so select and ensure that this is visible
              availableServiceList.clearSelection();
              serviceList[iServiceCount].selected = "true";
              availableServiceList.ensureElementIsVisible(serviceList[iServiceCount]);
              foundItem = true;
            }
          }
        }
        // Since the service we want is not in this area we need to close the folder
        // this way only the folder that has the service we want to highlight is open
        if (!foundItem) {
          folderList[iFolderCount].setAttribute("open", "false");
          folderList[iFolderCount].toggleOpen(null); 
        } else {
          folderList[iFolderCount].setAttribute("open", "true");
        }
      }
      return;
    }
  
    // We can make this better by actually moving the child elements arround rather than
    // redrawing the whole tree again.
    if (topic == "nsPref:changed") {
      // data will be the "show.account-setup-info-" + svc.shortName
      this.loadAvailableServices();
    }
    
    if(topic == "toggleAccountsArea") {
      this.openAccountArea(data);
    }
  },

  QueryInterface : function (aIID) {
    if ( !aIID.equals(Components.interfaces.nsISupports) &&
          !aIID.equals(Components.interfaces.nsIObserver) )
    {
      throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
  }

}

//! Initialize the sidebar controls for Accounts and Services.
/*!
  This function will be called when the Accounts and Services sidebar is loaded.
  It will load and group the available services in the Available Services tree control.
*/
accountsSidebar.onLoad = function () {
  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('accounts open');

  this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
  this.prefBranch.addObserver("", this, false);
                         
  this._logger = Components.classes['@flock.com/logger;1'].createInstance(Components.interfaces.flockILogger);
  this._logger.init('accountsSidebar');
  this._logger.info('starting up...');
  
  // Load the coop so we can use it later.
  this.coop = Components.classes["@flock.com/singleton;1"]
                        .getService(Components.interfaces.flockISingleton)
                        .getSingleton("chrome://browser/content/flock/common/load-faves-coop.js")
                        .wrappedJSObject;

  // So we can be localized
  this.strbundle = document.getElementById("accountStrings");

  // Get all the available services
  this.services = accountsSidebar.getFlockWebServices();

  // Load all available services
  this.loadAvailableServices();

  this.sidebarObserver = Components.classes["@mozilla.org/observer-service;1"]
                        .getService(Components.interfaces.nsIObserverService);
  this.sidebarObserver.addObserver(this, "accountsSidebar", false);
  this.sidebarObserver.addObserver(this, "toggleAccountsArea", false);
  // Set up observer for the flock-favorites
  var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                             .getService(Components.interfaces.nsIRDFService);

  var accountsDS = rdfService.GetDataSource("rdf:flock-favorites");
  accountsDS.AddObserver(accountsSidebar);
                            
  this.wsMgr = Components.classes["@flock.com/webservice-manager;1"]
                          .getService(Components.interfaces.flockIWebServiceManager);
		                          
}

//! Shutdown the sidebar controls for Accounts and Services.
/*!
  This function will be called when the Accounts and Services sidebar is unloaded.
  It will unregister observers.
*/
accountsSidebar.onUnLoad = function () {
  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount('accounts close');

  var rdfService = Components.classes["@mozilla.org/rdf/rdf-service;1"]
                             .getService(Components.interfaces.nsIRDFService);

  var accountsDS = rdfService.GetDataSource("rdf:flock-favorites");
  accountsDS.RemoveObserver(accountsSidebar);

  this.prefBranch.removeObserver("", this, false);

  this.sidebarObserver.removeObserver(this, "accountsSidebar");
  this.sidebarObserver.removeObserver(this, "toggleAccountsArea");
}

accountsSidebar.showPopup = function (data) {
  var popup = $("tipcontainer");
 
  // Remove old children 
  while(popup.firstChild) {
    popup.removeChild(popup.firstChild);
  }

  var text = document.createTextNode(data);
  var desc = document.createElement("description");
  desc.appendChild(text);
  popup.appendChild(desc);
  
  popup.setAttribute("height", desc.boxObject.height + 5);
}
//! Loads all available services.
/*!
  Load all available services in to the Available Services tree and group
  them by service type.
    Blogging
*/
accountsSidebar.loadAvailableServices = function () {
  // This should be turned into a xbl...
  // Populates the active service list
  var availableServiceList = $("availableServicesList");

  // Setup our groups
  var groupServiceArray = [];
  groupServiceArray["Media"] = [];
  groupServiceArray["Blog"] = [];
  groupServiceArray["Favorites"] = [];
  groupServiceArray["Disabled"] =[];

  // Clean the account list
  while (availableServiceList.getRowCount() > 0) {
    availableServiceList.removeItemAt(0);
  }

  // Loop through services to catagorize them.
  for (var i = 0; i < this.services.length; i++) {
    var contractID = this.services[i][0];
    var svc = this.services[i][1];

    var accountitem = document.createElement("richtreeitem");
    var accountLabel = document.createElement("label");
    var accountFavIcon = document.createElement("image");
    var accountBox = document.createElement("hbox");
    var tipData;

    // <richtreeitem value="contractID">
    //     <hbox onclick="">
    //       <image src="svc.icon" />
    //       <label value="Flickr" flex="1" tooltiptext="&flock.services.accountsSidebar.tooltips.XXXXX;" />
    //     </hbox>
    // </richtreeitem>

    accountitem.setAttribute('value', contractID);
    accountBox.setAttribute('flex', '1');
    accountFavIcon.setAttribute("src", svc.icon);
    accountLabel.setAttribute('value', svc.title);
    accountLabel.setAttribute('flex', '1');
    //accountLabel.setAttribute('onclick', 'accountsSidebar.gotoServiceLoginPage();');

    accountBox.appendChild(accountFavIcon);
    accountBox.appendChild(accountLabel);
    accountitem.appendChild(accountBox);

    // First we check if this service is disabled (the user has selected the
    // do not show me again checkbox from the notification bar.)
    if (svc.shortName != null) {
      //flock.notification.show.accounts.account-setup-info-flickr
      var prefName = SERVICE_SHOW_PREF_BRANCH + SERVICE_SHOW_PREF_HEADER + svc.shortName;
      if (this.prefService.getPrefType(prefName) &&
          !this.prefService.getBoolPref(prefName) &&
          svc.shortName != null)
      {
        accountitem.setAttribute('tooltip','aas-tooltip');
        tipData = this.strbundle.getFormattedString("flock.services.accountsSidebar.tooltips.disabled", [svc.title]);
        accountitem.setAttribute('onmouseover', "accountsSidebar.showPopup(\""+tipData+"\")");
        groupServiceArray["Disabled"].push(accountitem);
        continue;
      }

      // Set up the tooltip data
      accountitem.setAttribute('tooltip','aas-tooltip');
      try {
        tipData = this.strbundle.getString("flock.services.accountsSidebar.tooltips." + svc.shortName)
      } catch (err) {
        tipData = this.strbundle.getString("flock.services.accountsSidebar.tooltips.default");
      }
      accountitem.setAttribute('onmouseover', "accountsSidebar.showPopup(\""+tipData+"\")");

    }

			  
    // Note: Order of services checking here is important! If a service implements
    // more than 1 component (Ie: Blog and Photo) then the first one checked here
    // is where it will be catagorized.

    // Try and put it into the Blogging catagory
    // (Blogger, Blogsome, LiveJournal, Typepad, WordPress, Xanga...
    if (svc instanceof Components.interfaces.flockIBlogWebService) {
      groupServiceArray["Blog"].push(accountitem);
      continue;
    }

    // Next try the Media catagory (Flickr, Photobucket, YouTube...
    if (svc instanceof Components.interfaces.flockIMediaWebService) {
      groupServiceArray["Media"].push(accountitem);
      continue;
    }

    // Next try the bookmark catagory (del.icio.us, Magnolia, Shadows...)
    if (svc instanceof Components.interfaces.flockIBookmarkWebService) {
      groupServiceArray["Favorites"].push(accountitem);
      continue;
    }

  }

///////// Here is where we order the services into groups /////////

///////// BEGIN Media     //////////////
  var accountFolder = document.createElement("richtreefolder");
  accountFolder.setAttribute('title', this.strbundle.getString("flock.services.accountsSidebar.groups.media"));
  accountFolder.setAttribute('id', "flock.services.accountsSidebar.groups.media");
  accountFolder.setAttribute('open', 'true');
  if (groupServiceArray["Media"].length > 0) {
    accountFolder.setAttribute('open', 'true');
  }
  for (var i = 0; i < groupServiceArray["Media"].length; i++) {
    accountFolder.appendChild(groupServiceArray["Media"][i]);
  }
  availableServiceList.appendChild(accountFolder);
///////// END Media       //////////////

///////// BEGIN BLOGS //////////////
  var accountFolder = document.createElement("richtreefolder");
  accountFolder.setAttribute('title', this.strbundle.getString("flock.services.accountsSidebar.groups.blog"));
  accountFolder.setAttribute('id', "flock.services.accountsSidebar.groups.blog");
  accountFolder.setAttribute('open', 'true');
  if (groupServiceArray["Blog"].length > 0) {
    accountFolder.setAttribute('open', 'true');
  }
  for (var i = 0; i < groupServiceArray["Blog"].length; i++) {
    accountFolder.appendChild(groupServiceArray["Blog"][i]);
  }

  var accountitem = document.createElement("richtreeitem");
  var accountLabel = document.createElement("label");
  var accountSpacer = document.createElement("spacer");
  var accountBox = document.createElement("hbox");

  // Add the special service for self-hosted blog
  accountitem.setAttribute('value', 'CustomBlog');
  accountBox.setAttribute('flex', '1');
  accountSpacer.setAttribute('width', '16');
  accountSpacer.setAttribute('height', '16');
  accountLabel.setAttribute('value', this.strbundle.getString("flock.services.accountsSidebar.groups.manualBlog"));
  accountFolder.setAttribute('id', "flock.services.accountsSidebar.groups.manualBlog");
  accountLabel.setAttribute('flex', '1');
  accountLabel.setAttribute('tooltiptext', this.strbundle.getString("flock.services.accountsSidebar.tooltips.manualBlog"));
  accountBox.appendChild(accountSpacer);
  accountBox.appendChild(accountLabel);
  accountitem.appendChild(accountBox);
  accountFolder.appendChild(accountitem);
  availableServiceList.appendChild(accountFolder);
///////// END BLOGS       //////////////

///////// BEGIN Bookmarks //////////////
  var accountFolder = document.createElement("richtreefolder");
  accountFolder.setAttribute('title', this.strbundle.getString("flock.services.accountsSidebar.groups.bookmarks"));
  accountFolder.setAttribute('id', "flock.services.accountsSidebar.groups.bookmarks");
  accountFolder.setAttribute('open', 'true');
  if (groupServiceArray["Favorites"].length > 0) {
    accountFolder.setAttribute('open', 'true');
  }
  for (var i = 0; i < groupServiceArray["Favorites"].length; i++) {
    accountFolder.appendChild(groupServiceArray["Favorites"][i]);
  }
  availableServiceList.appendChild(accountFolder);
///////// END Bookmarks   //////////////

///////// BEGIN Disabled  //////////////
  var accountFolder = document.createElement("richtreefolder");
  accountFolder.setAttribute('title', this.strbundle.getString("flock.services.accountsSidebar.groups.disabled"));
   accountFolder.setAttribute('id', "flock.services.accountsSidebar.groups.disabled");
  if (groupServiceArray["Disabled"].length > 0) {
    accountFolder.setAttribute('open', 'true');
  }
  for (var i = 0; i < groupServiceArray["Disabled"].length; i++) {
    accountFolder.appendChild(groupServiceArray["Disabled"][i]);
  }
  availableServiceList.appendChild(accountFolder);
///////// END Disabled   //////////////
  
  
  if (groupServiceArray["Disabled"].length > 0) {
    // Open the disabled groups if there are any services currently disabled.
    this.openAccountArea("disabled");
  }
}

//! Loads all available services into an array.
/*!
  This will load all available services into an array of contractid and service object.
*/
accountsSidebar.getFlockWebServices = function () {
  var topMedia;
  this.prefService.getPrefType("flock.accounts.top.media") && (topMedia =
  this.prefService.getCharPref("flock.accounts.top.media"));
  
  var topBlog;
  this.prefService.getPrefType("flock.accounts.top.blog") && (topBlog =
  this.prefService.getCharPref("flock.accounts.top.blog"));

  var ignoreEntry;
  this.prefService.getPrefType("flock.accounts.ignore") && (ignoreEntry =
  this.prefService.getCharPref("flock.accounts.ignore"));

  var topFavorites;
  this.prefService.getPrefType("flock.accounts.top.favorite") && (topFavorites =
  this.prefService.getCharPref("flock.accounts.top.favorite"));
  
  var CATMAN = Components.classes['@mozilla.org/categorymanager;1'].getService(Components.interfaces.nsICategoryManager);
  var webServices = new Array();
  var e = CATMAN.enumerateCategory('flockWebService');
  while (e.hasMoreElements()) {
    try {
      var entry = e.getNext().QueryInterface(Components.interfaces.nsISupportsCString);
      if (!entry) { continue; }
      var contractID = CATMAN.getCategoryEntry('flockWebService', entry.data);
      var svc = Components.classes[contractID].getService(Components.interfaces.flockIWebService);

      // Check to see if we should NOT show this in the list
      var found = false;
      var ignoreList;
      if (ignoreEntry) {
        ignoreList = ignoreEntry.split(",");
        for each (var ignore in ignoreList) {
          if (svc.shortName.toLowerCase() == ignore.toLowerCase()) {
            found = true;
            break;
          }
        }
      }
      // Did we find something?
      if (found) {
        continue;
      }

      if ((svc.shortName == topMedia) || (svc.shortName == topBlog) || (svc.shortName == topFavorites)){
	    	webServices.unshift([contractID, svc]);
      } else {
        webServices.push([contractID, svc]);
      }
    } catch (e) {
      this._logger.info('Error occured in getFlockWebServices: ' + e);
    }
  }
  return webServices;
}

//! Go to a services login page.
/*!
  This will navigate to the defined login page for a service when the user
  double clicks on an entry in the Available services list.
*/
accountsSidebar.gotoServiceLoginPage = function () {
  if (!$('availableServicesList').selectedItem) {
    return;
  }

  var contractID = $('availableServicesList').selectedItem.value;

  switch (contractID) {
    case 'CustomBlog':
      // We need to open the custom blog editor instead
      window.openDialog("chrome://browser/content/flock/blog/blogAccountWizard.xul", "accountWizard", "chrome,modal,centerscreen", { defaultToCustom: true });
      $("activeAccountsList").builder.rebuild();
      break;
    case 'ApplyForService':
      // Here we just direct to a special url that the user can
      // request a service to add.
      // NOTE: Put this in a properties file!!! or better yet branding file
      var flockApplyForServiceUrl = "http://www.flock.com/";
      openUILinkIn(flockApplyForServiceUrl, "tab");
      break;
    default:
      var svc = Components.classes[contractID].getService(Components.interfaces.flockIWebService);
      if (svc) {
        var prefName = SERVICE_SHOW_PREF_BRANCH + SERVICE_SHOW_PREF_HEADER + svc.shortName;
        if (this.prefService.getPrefType(prefName) &&
            !this.prefService.getBoolPref(prefName))
        {
          // Re-enable this service
          this.prefService.setBoolPref(prefName, true);
        }
        // Tell the Web Service Manager to automatically keep any accounts
        // created for this service during the next while...
        this.wsMgr.autoKeepAccountForService(svc.urn);
        if (svc.redirUrl) {
          openUILinkIn(svc.redirUrl, "tab");
        } else {
          var redirURL = "http://redir.flock.com/redir?component=browser&partner="
                       + svc.shortName + "&action=accountssidebar_click";
          openUILinkIn(redirURL, "tab");
        }
      }
      break;
    }

}

//! Change Message
/*!
  This will change the message displayed directly above the Available Accounts
  we have it in a function so that if we want we can add timers to return to a
  default message.
*/
accountsSidebar.getNotificationBox = function () {
  var mainWindow = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                      .getInterface(Components.interfaces.nsIWebNavigation)
                      .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                      .rootTreeItem
                      .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                      .getInterface(Components.interfaces.nsIDOMWindow);
  var gBrowser = mainWindow.getBrowser();
  var notificationBox = gBrowser.getNotificationBox();

  return notificationBox;
}

accountsSidebar.clearNotifications = function (aNotificationList) {
  var notificationBox = this.getNotificationBox();

  for (var iIndex = 0; iIndex < aNotificationList.length; iIndex++) {
    var oldNotification = notificationBox.getNotificationWithValue(aNotificationList[iIndex]);
    if (oldNotification) {
      notificationBox.removeNotification(oldNotification);
    }
  }
}


//! Forget Account
/*!
  This will remove the selected account from the browser, prompting the user
  before doing so.
*/
accountsSidebar.forgetAccount = function () {
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Components.interfaces.flockIAccountUtils);

  var currentAccount = $("activeAccountsList").selectedItem;
  if (currentAccount.getAttribute("keepingaccount") == "true") {
    // We are in the process of keeping this account so don't allow for double clicks
    return;
  }

  var id = currentAccount.id;
  var account = this.coop.get(id);
  var svc = Components.classes[account.serviceId].getService(Components.interfaces.flockIWebService);

  var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].getService(Components.interfaces.nsIPromptService);

  if (prompts.confirm(window, 
                   this.strbundle.getString("flock.services.accountsSidebar.remove.title") ,
                   this.strbundle.getString("flock.services.accountsSidebar.remove.msg"))) {
    svc.getAccount(id).logout(null);
    svc.removeAccount(id);

    this.sidebarObserver.notifyObservers(null, 'refresh-myworld-accounts', false);
    this.clearNotifications(["account-setup-success-" + svc.shortName, "account-setup-info-" + svc.shortName]);

    var notificationBox = this.getNotificationBox();
    var message = this.strbundle.getFormattedString("flock.services.accountsSidebar.removedAccount", [svc.title]);
    notificationBox.appendNotification( message,
      "account-remove-success-" + svc.shortName,
      "chrome://browser/skin/Info.png",
      notificationBox.FLOCK_PRIORITY_MEDIUM, null, null );
  }
}

//! Keep Account
/*!
  When a Transient account is selected this will allow the user to keep said
  account after restarts of the browser.
*/
accountsSidebar.keepAccount = function () {
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Components.interfaces.flockIAccountUtils);
  var SB = "common/notificationOverlay";
  var currentAccount = $("activeAccountsList").selectedItem;
  if (currentAccount.getAttribute("keepingaccount") == "true") {
    // We are in the process of keeping this account so don't allow for double clicks
    return;
  }

  var id = currentAccount.id;
  var account = this.coop.get(id);
  var svc = Components.classes[account.serviceId].getService(Components.interfaces.flockIWebService);
  svc.getAccount(id).keep();
  var metrics = Components.classes['@flock.com/metrics-service;1']
    .getService(Components.interfaces.flockIMetricsService);
  metrics.reportCount("keep "+svc.shortName+" account");
  var message = null;
  var link = null;
  var appName = brandGetString("brand", "brandShortName");
  
  if(svc instanceof Components.interfaces.flockIBookmarkWebService)
  {
    link = [{
      label: flockGetString(SB, "favs.bar1.link1"),
      callback: function () {
        toggleSidebar("viewBookmarksSidebar", true);
      }
    }];
    message = flockGetString(SB, "favs.bar2.label");
    
  }else if(svc instanceof Components.interfaces.flockIMediaWebService)
  {
     message = flockGetString(SB, "media.bar2.label", [appName, svc.title]);
    
  }else if(svc instanceof Components.interfaces.flockIBlogWebService)
  {
    message = flockGetString(SB, "blog.bar2.label", [svc.title]);
    link = [{
      label: flockGetString(SB, "blog.bar2.link1"),
      callback: function () {
        flock_blogNewPost();
      }
    }];
    
  }else
  {
    message = this.strbundle.getFormattedString("flock.services.accountsSidebar.configuredAccount", [svc.title]);
  }
  var favicon = "chrome://browser/skin/Info.png";
  try { favicon = document.getElementById("page-proxy-favicon").src; }
  catch (ex) { dump(ex +'\n')};
  
  var notificationBox = this.getNotificationBox();
  notificationBox.removeAllNotifications();
  notificationBox.appendUniqueNotification( message,
    "account-setup-success-" + svc.shortName,
    favicon,
    notificationBox.FLOCK_PRIORITY_MEDIUM, null, link );
}

//! Navigate to account page.
/*!
  Opens the selected accounts home page in a new tab.
*/
accountsSidebar.navigateToAccountPage = function (accountPageUrlLabel) {
  openUILinkIn(accountPageUrlLabel.getAttribute("urlLink"), "tab");
}

//! Log into a service
/*!
 This will just navigate to the service page
*/
accountsSidebar.logIn = function () {
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Components.interfaces.flockIAccountUtils);

  var currentAccount = $("activeAccountsList").selectedItem;
  var id = currentAccount.id;
  var contractID = acUtils.getServiceIDForAccountURN(id);  // @flock.com/people/wordpress;1
  var svc = Components.classes[contractID].getService(Components.interfaces.flockIWebService);

  openUILinkIn(svc.url, "tab");
}

//! openBlogSettings
/*!
 This will open the blogOptions dialog for a self-hosted blog.
*/
accountsSidebar.openBlogSettings = function(account) {
  var param = {
    accountId: account.parentNode.id
  }
  window.openDialog("chrome://browser/content/flock/blog/blogAccountDlg.xul", "editCustomBlog", "modal,centerscreen", param); 
}

accountsSidebar.logout =
function accountsSidebar_logout() {
  this._logger.debug(".logout()");
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Components.interfaces.flockIAccountUtils);
  var currentAccount = $("activeAccountsList").selectedItem;
  var id = currentAccount.id;
  var contractId = acUtils.getServiceIDForAccountURN(id);
  var svc = Components.classes[contractId]
                      .getService(Components.interfaces.flockIWebService);
  svc.logout();
  acUtils.markAllAccountsAsLoggedOut(contractId);
}

accountsSidebar.openAccountArea = function(aArea)
{
  var areas =  this.gAccountsArea;
 for(var i = 0; i<areas.length; i++)
 {
   if(areas[i] == aArea)
   {
     document.getElementById('flock.services.accountsSidebar.groups.'+aArea).setAttribute('open', 'true');
   }else
   {
     document.getElementById('flock.services.accountsSidebar.groups.'+areas[i]).setAttribute('open', 'false');
   }
 }
}

accountsSidebar.trackSelected = function() {
  var accountList = $('activeAccountsList');
  var curSelected = accountList.selectedItem;
  if (curSelected == accountsSidebar.savedSelected) {
    // Deselect the account
    accountList.clearSelection();
    accountsSidebar.savedSelected = null;
  } else {
    accountsSidebar.savedSelected = curSelected;
  }
}

accountsSidebar.openBlogSettings =
function accountsSidebar_openBlogSettings(aAccountId) {
  var acUtils = Components.classes["@flock.com/account-utils;1"]
                          .getService(Components.interfaces.flockIAccountUtils);
  var currentAccount = $("activeAccountsList").selectedItem;
  var id = currentAccount.id;
  var win = window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                  .getInterface(Components.interfaces.nsIWebNavigation)
                  .QueryInterface(Components.interfaces.nsIDocShellTreeItem)
                  .rootTreeItem
                  .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
                  .getInterface(Components.interfaces.nsIDOMWindow);

  win.openDialog("chrome://browser/content/flock/blog/blogAccountDlg.xul",
                 "editCustomBlog",
                 "modal,centerscreen",
                 { accountId: id });
}
