// vim: ts=2 sw=2 expandtab cindent
//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function componentToolkit()
{
  this._logger = Cc["@flock.com/logger;1"].createInstance(Ci.flockILogger);
  this._logger.init("componentToolkit");
  this._logger.info("component toolkit object created");
}

componentToolkit.prototype.test =
function componentToolkit_test()
{
  this._logger.info("test!");
}

componentToolkit.prototype.addAllInterfaces =
function componentToolkit_addAllInterfaces(aComponent)
{
  this._logger.info(".addAllInterfaces('"+aComponent._ctk.shortName+"')");
  for (var i in aComponent._ctk.interfaces) {
    this.addInterface(aComponent, aComponent._ctk.interfaces[i]);
  }
}

componentToolkit.prototype.getACUtils =
function componentToolkit_getACUtils(aComponent)
{
  if (!aComponent.acUtils) {
    aComponent.acUtils = Cc["@flock.com/account-utils;1"]
      .getService(Ci.flockIAccountUtils);
  }
  return aComponent.acUtils;
}

componentToolkit.prototype.getWD =
function componentToolkit_getWD(aComponent)
{
  if (!aComponent.webDetective) {
    aComponent.webDetective =
      this.getACUtils(aComponent)
          .useWebDetective(aComponent._ctk.shortName.toLowerCase()+".xml");
  }
  return aComponent.webDetective;
}

componentToolkit.prototype.newResults =
function componentToolkit_newResults()
{
  return Cc["@mozilla.org/hash-property-bag;1"]
    .createInstance(Ci.nsIWritablePropertyBag2);
}

componentToolkit.prototype.getCoop =
function componentToolkit_getCoop(aComponent)
{
  if (!aComponent._coop) {
    aComponent._coop = Cc["@flock.com/singleton;1"]
      .getService(Ci.flockISingleton)
      .getSingleton("chrome://browser/content/flock/common/load-faves-coop.js")
      .wrappedJSObject;
  }
  return aComponent._coop;
}

componentToolkit.prototype.addInterface =
function componentToolkit_addInterface(aComponent, aInterface)
{
  if (!aInterface) return;
  // Add aInterface to aComponent's interface list, if it isn't already there
  if (!aComponent._ctk) { aComponent._ctk = {}; }
  var _ctk = aComponent._ctk;
  if (!_ctk.interfaces) { _ctk.interfaces = []; }
  for (var i = 0; i < _ctk.interfaces.length; i++) {
    if (_ctk.interfaces[i] == aInterface) { break; }
  }
  if (i < _ctk.interfaces.length) { _ctk.interfaces.push(aInterface); }

  var proto = aComponent.__proto__;
  var ctk = this;

  switch (aInterface) {

    case "nsISupports":
    {
      if (!proto.QueryInterface) {
        proto.QueryInterface =
        function QueryInterface(aIID)
        {
          var ifaces = this._ctk.interfaces;
          for (var i in ifaces) {
            if (aIID.equals(Ci[ifaces[i]])) { return this; }
          }
          throw Cr.NS_ERROR_NO_INTERFACE;
        };
      }
    }; break;

    case "nsISupportsCString":
    {
      if (!proto.toString) {
        proto.toString =
        function toString()
        {
          return this._ctk.shortName;
        };
      }
    }; break;

    case "nsIClassInfo":
    {
      if (!proto.flags) {
        proto.flags = Ci.nsIClassInfo.SINGLETON;
      }
      if (!proto.implementationLanguage) {
        proto.implementationLanguage = Ci.nsIProgrammingLanguage.JAVASCRIPT;
      }
      if (!proto.classID) {
        proto.classID = aComponent._ctk.CID;
      }
      if (!proto.classDescription) {
        proto.classDescription = aComponent._ctk.description;
      }
      if (!proto.contractId) {
        proto.contractId = aComponent._ctk.contractID;
      }
      if (!proto.getInterfaces) {
        proto.getInterfaces =
        function getInterfaces(aCount)
        {
          var iids = [];
          var ifaces = this._ctk.interfaces;
          for (var i in ifaces) { if (ifaces[i]) iids.push(Ci[ifaces[i]]); }
          aCount.value = iids.length;
          return iids;
        };
      }
      if (!proto.getHelperForLanguage) {
        proto.getHelperForLanguage =
        function getHelperForLanguage(aLanguage)
        {
          return null;
        };
      }
    }; break;

    case "nsIObserver":
    {
      if (!proto.observe) {
        proto.observe = function observe(aSubject, aTopic, aData) {};
      }
    }; break;

    case "flockIWebService":
    {
      if (!proto.urn) {
        proto.urn = "urn:"+aComponent._ctk.shortName.toLowerCase()+":service";
      }
      if (!proto.shortName) {
        proto.shortName = aComponent._ctk.shortName;
      }
      if (!proto.title) {
        proto.title = aComponent._ctk.fullName;
      }
      if (!proto.icon) {
        proto.icon = aComponent._ctk.favicon;
      }
      if (proto.needPassword == undefined) {
        if (aComponent._ctk.needPassword == undefined) {
          proto.needPassword = true;
        } else {
          proto.needPassword = aComponent._ctk.needPassword;
        }
      }
      if (!proto.getAccount) {
        proto.getAccount =
        function getAccount(aURN)
        {
          var c_acct = ctk.getCoop(this).get(aURN);
          var acct = new aComponent._ctk.accountClass();
          acct.urn = c_acct.id();
          acct.username = c_acct.name;
          return acct;
        };
      }
      if (!proto.getAccounts) {
        proto.getAccounts =
        function getAccounts()
        {
          return ctk.getACUtils(this)
                    .getAccountsForService(aComponent._ctk.contractID);
        };
      }
      if (!proto.removeAccount) {
        proto.removeAccount =
        function removeAccount(aURN)
        {
          ctk.getACUtils(this).removeAccount(aURN);
        };
      }
      if (!proto.logout) {
        proto.logout =
        function logout()
        {
          var cookies = ctk.getWD(this)
                           .getSessionCookies(aComponent._ctk.shortName);
          if (cookies) {
            ctk.getACUtils(this).removeCookies(cookies);
          }
        };
      }
    }; break;

    case "flockIWebServiceAccount":
    {
      if (!proto.activate) {
        proto.activate =
        function activate(aListener)
        {
          var c_acct = ctk.getCoop(this).get(this.urn);
          c_acct.isPollable = true;
          if (aListener) {
            aListener.onSuccess(this, "activate");
          }
        };
      }
      if (!proto.deactivate) {
        proto.deactivate =
        function deactivate(aListener)
        {
          var c_acct = ctk.getCoop(this).get(this.urn);
          c_acct.isPollable = false;
          if (aListener) {
            aListener.onSuccess(this, "deactivate");
          }
        };
      }
      if (!proto.login) {
        proto.login =
        function login(aListener)
        {
          if (aListener) {
            aListener.onError(this, "login", null);
          }
        };
      }
      if (!proto.logout) {
        proto.logout =
        function logout(aListener)
        {
          var c_acct = ctk.getCoop(this).get(this.urn);
          if (c_acct.isAuthenticated) {
            c_acct.isAuthenticated = false;
            Cc[c_acct.serviceId].getService(Ci.flockIWebService).logout();
          }
          if (aListener) {
            aListener.onSuccess(this, "logout");
          }
        };
      }
      if (!proto.keep) {
        proto.keep =
        function keep()
        {
          var c_acct = ctk.getCoop(this).get(this.urn);
          c_acct.isTransient = false;
          var svc = Cc[c_acct.serviceId].getService(Ci.flockIWebService);
          if (svc instanceof Ci.flockIPollingService) {
            c_acct.nextRefresh = new Date(0);
          }
          ctk.getACUtils(this).makeTempPasswordPermanent(svc.urn+":"+c_acct.accountId);
        };
      }
      if (!proto.remove) {
        proto.remove =
        function remove()
        {
          var c_acct = ctk.getCoop(this).get(this.urn);
          var svc = Cc[c_acct.serviceId].getService(Ci.flockIWebService);
          svc.removeAccount(this.urn);
        };
      }
    }; break;

    case "flockIManageableWebService":
    {
      if (!proto.ownsLoginForm) {
        proto.ownsLoginForm =
        function ownsLoginForm(aForm)
        {
          aForm.QueryInterface(Ci.nsIDOMHTMLFormElement);
          var wd = ctk.getWD(this);
          var shortName = aComponent._ctk.shortName.toLowerCase();
          if (wd.detectForm(shortName, "login", aForm, ctk.newResults())) {
            return true;
          }
          if (wd.detectForm(shortName, "signup", aForm, ctk.newResults())) {
            return true;
          }
          if (wd.detectForm(shortName, "changepassword", aForm, ctk.newResults())) {
            return true;
          }
          return false;
        };
      }
      if (!proto.getCredentialsFromForm) {
        proto.getCredentialsFromForm =
        function getCredentialsFromForm(aForm)
        {
          aForm.QueryInterface(Ci.nsIDOMHTMLFormElement);
          var wd = ctk.getWD(this);
          var shortName = aComponent._ctk.shortName.toLowerCase();
          var formType = "login";
          var results = ctk.newResults();
          if (!wd.detectForm(shortName, formType, aForm, results)) {
            formType = "signup"
            results = ctk.newResults();
            if (!wd.detectForm(shortName, formType, aForm, results)) {
              formType = "changepassword"
              results = ctk.newResults();
              if (!wd.detectForm(shortName, formType, aForm, results)) {
                results = null;
              }
            }
          }
          if (results) {
            return {
              QueryInterface: function(aIID) {
                if (!aIID.equals(Components.interfaces.nsISupports) &&
                    !aIID.equals(Components.interfaces.nsIPassword) &&
                    !aIID.equals(Components.interfaces.flockIPasswordOrigin)) { 
                  throw Components.interfaces.NS_ERROR_NO_INTERFACE;
                }
                return this;
              },
              host: "",
              user: results.getPropertyAsAString("username"),
              password: results.getPropertyAsAString("password"),
              formType: formType
            };
          }
          return null;
        };
      }
      if (!proto.ownsDocument) {
        // Note that ownsDocument() only gets called for documents hosted in a
        // domain for this service... so if no further tests are necessary,
        // it's ok to just return 'true' here.
        proto.ownsDocument =
        function ownsDocument(aDocument)
        {
          return true;
        };
      }
      if (!proto.docRepresentsSuccessfulLogin) {
        proto.docRepresentsSuccessfulLogin =
        function docRepresentsSuccessfulLogin(aDocument)
        {
          return ctk.getWD(this).detect( aComponent._ctk.shortName.toLowerCase(),
                                         "loggedin", aDocument, null );
        };
      }
      if (!proto.getAccountIDFromDocument) {
        proto.getAccountIDFromDocument =
        function getAccountIDFromDocument(aDocument)
        {
          var results = ctk.newResults();
          if (ctk.getWD(this).detect( aComponent._ctk.shortName.toLowerCase(),
                                      "accountinfo", aDocument, results ))
          {
            return results.getPropertyAsAString("accountid");
          }
          return null;
        };
      }
    }; break;

    case "flockISocialWebService":
    {
      if (!proto.decorateForPerson) {
        proto.decorateForPerson = function decorateForPerson(aDocument) { };
      }
    }; break;

    case "flockISocialWebServiceAccount":
    {
      if (!proto.browseFriends) {
        proto.browseFriends = function browseFriends(aFriendURN, aListener) { };
      }
    }; break;

    case "flockIMediaWebService":
    {
      if (!proto.decorateForMedia) {
        proto.decorateForMedia = function decorateForMedia(aDocument) { };
      }
      if (!proto.handlesMediaStream) {
        proto.handlesMediaStream = function handlesMediaStream() {
          return false;
        }
      }
      if (!proto.checkIsStreamUrl) {
        proto.checkIsStreamUrl = function checkIsStreamUrl(aUrl) {
          return false;
        }
      }
      if (!proto.getMediaQueryFromURL) {
        proto.getMediaQueryFromURL = function getMediaQueryFromURL(aURL, aListener) {
          aListener.onError(null, "Media service is not supported.", null);
        }
      }
    }; break;

    default:
      this._logger.warn("Unhandled interface: "+aInterface);
  }
}


// cheap scheduler for javascript tasks

// aSlice is the maximum timeslice in seconds. 0.5 or less is a good value

// aPercent is the percent of wall-clock time that should be spent executing
// the task - eg: 20 = one fifth of the time

// aTask is defined to be a generator taking an function that evaluates
// to true if it should yield like:
// function task (should_yield) {
//   while (some_condition) {
//     do some work...
//     if (should_yield()) yield;
//   }
// }
// NOTE: should_yield() is fairly cheap but in a tight inner loop it might
//       make sense to call it every N cycles
componentToolkit.prototype.schedule =
function componentToolkit_schedule(aTimers, aSlice, aPercent, aTask)
{
  // convert percent to ratio
  var ratio = aPercent / 100.0;

  // convert slice to milliseconds
  var slice = aSlice * 1000.0;

  function milliseconds() { return (new Date ()).getTime(); }
  var wall_start = milliseconds();
  var process_time = 0;
  var process_start = 0;

  function should_yield() {
    var now = milliseconds();
    var so_far = now - process_start;
    var wall_time = now - wall_start;
    return (so_far >= slice) || ((wall_time * ratio) < (process_time + so_far));
  }

  var generator = aTask(should_yield);
  var timer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);

  var callback = {
    notify: function () {
      var wall_time = milliseconds() - wall_start;

      if ((wall_time * ratio) < process_time) {
        // if we've spent more time than we're supposed to
        // don't run this cycle
      }

      process_start = milliseconds();
      try {
        generator.next();
      } catch (err if err instanceof StopIteration) {
        // the task is complete
        timer.cancel();
        for (var i = 0; i < aTimers.length; i++) {
          if (aTimers[i] == timer) {
            aTimers.splice(i, 1);
          }
        }
        delete timer;
      }
      process_time += (milliseconds() - process_start);
    }
  };

  timer.initWithCallback( callback,
                          Math.round(slice/ratio),
                          Ci.nsITimer.TYPE_REPEATING_SLACK );
  aTimers.push(timer);
}
