//
// BEGIN FLOCK GPL
// 
// Copyright Flock Inc. 2005-2007
// http://flock.com
// 
// This file may be used under the terms of of the
// GNU General Public License Version 2 or later (the "GPL"),
// http://www.gnu.org/licenses/gpl.html
// 
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
// 
// END FLOCK GPL
//

/**
 Flock mediaStream object, single instance of the object for each browser
 */
var mediaStream = {
  service: null,
  refUrl: "",
  html: "",
  eventType: "",
  mTimer: null
}

/**
 These are some compoenents that the mediaStream uses to operate.
 */
mediaStream.mPhotoAPIManager = Components.classes['@flock.com/photo-api-manager;1?']
                                         .getService(Components.interfaces.flockIPhotoAPIManager);
mediaStream.mPrefSvc = Components.classes['@mozilla.org/preferences-service;1']
                                 .getService(Components.interfaces.nsIPrefBranch);
mediaStream._logger = Components.classes['@flock.com/logger;1']
                                .createInstance(Components.interfaces.flockILogger);


/**
 onclick event handler for the mediaStream overlay
 this will open the mediaStream for the image under the mouse in
 the media bar.
 */
mediaStream.onclick = function(event) {
    mediaStream._logger.debug("onclick called");
    if (mediaStream.service == null) return;
    
    $('photo-stream-menu').setAttribute('hidden', true);    
    $('video-stream-menu').setAttribute('hidden', true);    

    var mediaBarQueryListener = {
      onSuccess:function (aResults, aTopic) {
        // aResults is a bag of (query, title)
        if (mediabar.lock()) {
          aResults = aResults.QueryInterface(Components.interfaces.nsIPropertyBag2);
          var query = aResults.getPropertyAsAString("query");
          var title = aResults.getPropertyAsAString("title");
          mediabar.loadQuery(mediaStream.service.shortName, query, title);
          mediabar.unlock();
        }
        mediaStream.hide();
      },
      onError:function(aSubject, aTopic, aError) {
        mediaStream.hide();
        mediaStream._logger.debug(aTopic);
      }
    }
    
    
    mediaStream._logger.debug("Event Type == " + event);
    var metrics = Components.classes['@flock.com/metrics-service;1'].getService(Components.interfaces.flockIMetricsService);
    if (event == 'stream') {
      metrics.reportCount('mediabar from flockMediaStream');
      mediaStream.service.getMediaQueryFromURL(mediaStream.refUrl, mediaBarQueryListener);
    }
    
    mediaStream._logger.debug("HTML is [" + mediaStream.html + "]");
    
    if (event == 'blog') {
      metrics.reportCount('blog from flockMediaStream');
      flock.blog.service.openEditor(null, this.html, null);
      mediaStream.hide();
    }
    
    if (event == 'email') {
      metrics.reportCount('email from flockMediaStream');
      var mSubject = mediaStream.bundle.GetStringFromName("flock.services.mediastream.email.subject");
      var mBody = mediaStream.bundle.GetStringFromName("flock.services.mediastream.email.body");
      if (mediaStream.eventType == "image") {
        mBody += "%0A" + this.refUrl;
      } else {
        mBody += "%0A" +  document.getElementById("urlbar").value;
      }
      mBody += "%0A%0A" + mediaStream.bundle.GetStringFromName("flock.services.mediastream.email.getFlock");
      
      var url = "mailto:?";
      url += "subject=" + mSubject;
      url += "&body=" + mBody;
      window.openUILink(url);
      mediaStream.hide();
    }
}

mediaStream.saveCords = function(event) {
  mediaStream.curMouseX = event.clientX;
  mediaStream.curMouseY = event.clientY;
}

/**
  gets the desktop cordinates of the image or embed we are overlaying...me
  */
mediaStream.getBoxPos = function(obj) {
  if (obj == null) return [0,0,0,0];
  
  var scrollTop = obj.ownerDocument.body.scrollTop;
  if (!scrollTop) {
      scrollTop = obj.ownerDocument.documentElement.scrollTop;
  }

  var scrollLeft = obj.ownerDocument.body.scrollLeft;
  if (!scrollLeft) {
      scrollLeft = obj.ownerDocument.documentElement.scrollLeft;
  }
      
  var top = 0, left = 0;
  var temp=obj;
  while (temp) {
    top += temp.offsetTop;
    left += temp.offsetLeft;
    temp = temp.offsetParent;
  }
  
  top = top - scrollTop;
  left = left - scrollLeft;
  bottom = top + parseInt(obj.height);  // Need to use parseInt here for embed objects as they are strings not ints
  right = left + parseInt(obj.width);
  
  return [top, left, bottom, right];
}

/**
 show this will show the mediaStream overlay when the user mouses over
 and image or video.
 */
mediaStream.show = function(obj , service, refUrl, html, eventType) {
    mediaStream._logger.debug("show called");
    
    if ((obj.height <= 125) && (obj.width <= 250)) return;
    
    mediaStream.service = service;
    mediaStream.refUrl = refUrl;
    mediaStream.html = html;
    mediaStream.eventType = eventType;
    
    var scrollTop = obj.ownerDocument.body.scrollTop;
    if (!scrollTop) {
        scrollTop = obj.ownerDocument.documentElement.scrollTop;
    }

    var scrollLeft = obj.ownerDocument.body.scrollLeft;
    if (!scrollLeft) {
        scrollLeft = obj.ownerDocument.documentElement.scrollLeft;
    }
        
    var top = 0, left = 0;
    var temp=obj;
    while (temp) {
        top += temp.offsetTop;
        left += temp.offsetLeft;
        temp = temp.offsetParent;
    }
    
    // have to hard code the width and height of the mediaStream box since it may or may not have them yet.
    var mediaStreamBoxHeight = 66 + $('status-bar').boxObject.height;
    var mediaStreamBoxWidth = 241;
    
    // only show if the picture is fully visible
    if(top < scrollTop) return;
    if(left < scrollLeft) return;
    
    // we need to add the tabHeight to the y position of the mediaStream overlay
    var tabHeight = document.getAnonymousElementByAttribute(gBrowser, "class", "tabbrowser-strip").boxObject.height;

    // we need to add notification heights to the y position of the mediaStream overlay
    var notificationsHeight = 0;
    var notificationBox = gBrowser.getNotificationBox();
    if(notificationBox) {
      var notifications = notificationBox.allNotifications;
      for (var i = 0; i < (notifications.length); i++) {    // for some reason notifications.length is one more than there really is.
        notificationsHeight += notifications[i].boxObject.height;
        var mTop = parseInt(notifications[i].style.marginTop);
        if (mTop > 0) notificationsHeight += mTop;
      }
    }

    var view = obj.ownerDocument.defaultView;
    var style = view.getComputedStyle(obj, "");

    var borderLeft = (parseInt(style.getPropertyCSSValue('border-left-width').cssText) || 0) +
                     (parseInt(style.getPropertyCSSValue('margin-left').cssText) || 0) +
                     (parseInt(style.getPropertyCSSValue('padding-left').cssText) || 0); // XXX - this assumes pixels!
    var borderTop  = (parseInt(style.getPropertyCSSValue('border-top-width').cssText) || 0) +
                     (parseInt(style.getPropertyCSSValue('margin-top').cssText) || 0) +
                     (parseInt(style.getPropertyCSSValue('padding-top').cssText) || 0); // XXX - this assumes pixels!

    var pmTop  = top + $('content').boxObject.y + tabHeight + notificationsHeight + borderTop - scrollTop;
    var pmLeft = left + $('content').boxObject.x - scrollLeft + borderLeft;

    //if (pmTop + 24 > ($('content').boxObject.y + $('content').boxObject.height)) return;
    // Don't show if we are going to go off the bottom or left of the browser.    
    if (pmTop > (document.height - mediaStreamBoxHeight)) return;
    if (pmLeft > (document.width - mediaStreamBoxWidth)) return;
    
    mediaStream.targetObj = obj;
    $('media-stream').style.top = pmTop + 'px';
    $('media-stream').style.left = pmLeft  + 'px';
    $('media-stream').removeAttribute('hidden');
}

mediaStream.mouseover = function() {
    mediaStream._logger.debug("mouseover called");
    // We need to expand the menu based on what type of object this points to
    if (this.eventType == 'image') {
      $('photo-stream-menu').removeAttribute('hidden');    
    } else {
      $('video-stream-menu').removeAttribute('hidden');    
    }
    if (mediaStream.mTimer) {
      clearTimeout(mediaStream.mTimer);
      mediaStream.mTimer = null;
    }
}

mediaStream.mouseout = function(event) {
    mediaStream._logger.debug("mouseout called");
    $('photo-stream-menu').setAttribute('hidden', true);    
    $('video-stream-menu').setAttribute('hidden', true);    
    $('error-message-menu').setAttribute('hidden', true);
    
    mediaStream.hide();
}

/*! This is the final and definite hide function, it will hide the overlay and
    reset all the variables.
 */
mediaStream.forceHide = function() {
  mediaStream._logger.debug("hide called");    
  $('media-stream').setAttribute('class', '');
  $('media-stream').setAttribute('hidden', true);
  $('photo-stream-menu').setAttribute('hidden', true);    
  $('video-stream-menu').setAttribute('hidden', true);    
  $('error-message-menu').setAttribute('hidden', true);
  
  mediaStream.service = null;
  mediaStream.refUrl = "";
  mediaStream.html = "";
  clearTimeout(mediaStream.mTimer);
  mediaStream.mTimer = null;
  mediaStream.targetObj = null;
}

/*! This hide is called from hide after a timeout, here we check if the
    mouse is really not in the objects boundries and if not we hide calling
    forceHide function.
 */
mediaStream.realHide = function() {
  if (mediaStream.checkIfMouseInObject(mediaStream.targetObj)) {
    return;
  }
  mediaStream.forceHide();    
}  

/*! This first hide sets a timeout to call the realHide function
    we put the timeout so that we don't get conflicting calls which
    would cause flickering.
 */
mediaStream.hide = function() {
  mediaStream.mTimer = setTimeout("mediaStream.realHide()", 250);
}

mediaStream.checkIfMouseInObject = function(checkObject) {  
  var objPos = mediaStream.getBoxPos(checkObject);
  
  // Find mouse pos...
  var mX = mediaStream.curMouseX;
  var mY = mediaStream.curMouseY;

  if (objPos[0] == 0 && objPos[1] == 0 && objPos[2] == 0 && objPos[3] == 0) {
    return false;
  }
  
  if (mY > objPos[0] && mY < objPos[2] &&
    mX > objPos[1] && mX < objPos[3]) {
    return true;
  }
  return false;
}

mediaStream.imagerOut = function(event) {
  // First we have to find out what the box cordinates of the image/embed object are
  // Then we translate those into cordinates that will work with the event.clientX/clientY cordinates
  if (event.target.nodeName == 'IMG' ||
      event.target.nodeName == 'EMBED') {
    
    // Ok we are probably mousing out of the image/embed so lets check the cordinates
    //if (!mediaStream.checkIfMouseInObject(event.target, event)) {
      mediaStream.hide();
    //} 
  }
}

mediaStream.imager = function(event) {
  // Ignore other objects.
  if (event.target.nodeName != 'IMG' &&
      event.target.nodeName != 'EMBED') {
    return;
  }
  
  // Ignore if media-stream is already visible
  if ($('media-stream').getAttribute('hidden') == null) {
    return;
  }
  
  if (event.target.nodeName == 'IMG') {
    mediaStream._logger.debug("found an IMG, checking handlers...");
    var useUrl = event.target.parentNode.href;
    
    // If we are not wrapped in an Anchor then check the image source
    if (event.target.parentNode.nodeName != "A") {
      mediaStream._logger.debug("Searching fro proper url since we don't have an A tag...");
      // http://l.yimg.com/www.flickr.com/images/spaceball.gif
      // http://farm1.static.flickr.com/191/518298657_f288340c49.jpg?v=0
      if (event.target.src.match(/^http:\/\/l\.yimg\.com\/www\.flickr\.com\/images\/spaceball\.gif/) ||
          event.target.src.match(/^https?:\/\/farm\d+\.static\.flickr\.com\/.*/)) {
        mediaStream._logger.debug("This is a Flickr image, so we need to grab the current url...");
        useUrl = document.getElementById("urlbar").value;
      } else { 
        mediaStream._logger.debug("Not a special image, so just use the image source...");
        useUrl = event.target.src;
      }
    } else if (event.target.parentNode.href.match(/^javascript:void\(0\)/)) {
        // This could be a photobucket image or some other one wrapped in a javascript <a tag
        mediaStream._logger.debug("Using the image tag source...");
        useUrl = event.target.src;
    } else {
      mediaStream._logger.debug("Using the A tag source...");
    }
    mediaStream._logger.debug("useUrl = [" + useUrl + "]");
    
    for(var i=0;i<mediaStream.mediaWebServices.length;++i) {
      mediaStream._logger.debug("checking service: " + mediaStream.mediaWebServices[i].title);
      var results = mediaStream.mediaWebServices[i].checkIsStreamUrl(useUrl);
      if (results) {
        var html = event.target.parentNode.innerHTML;
        if (event.target.parentNode.href)
          html = "<a href='"+event.target.parentNode.href+"'>"+html+"</a>";
        mediaStream.show(event.target, mediaStream.mediaWebServices[i], useUrl, html, 'image');
        break;
      }
    }
  } else if (event.target.nodeName == 'EMBED' && event.target.type == 'application/x-shockwave-flash') {
    mediaStream._logger.debug("found an EMBED for shockwave-flash, checking handlers...");
    var useUrl = document.getElementById("urlbar").value;
    for(var i=0;i<mediaStream.mediaWebServices.length;++i) {
      mediaStream._logger.debug("checking service: " + mediaStream.mediaWebServices[i].title);
      var results = mediaStream.mediaWebServices[i].checkIsStreamUrl(event.target.src);
      if (results) {
        var html;
        if (event.target.src.match(/\/player\w*\.swf\?v*/)) {
          // ERWN: evil hack for videos directly from youtube.com
          var re1 = /\/player2\.swf\?.*video_id=(.+)&/;
          var re2 = /video_id=(.+)&.*&/;
          var arr = [];
          if (event.target.src.match(re1)) {
            arr = re1.exec(event.target.src);
          } else if (event.target.getAttribute("flashvars").match(re2)) {
            arr = re2.exec(event.target.getAttribute("flashvars"));
            event.target.src = "/player2.swf?video_id=" + arr[1];
          }
          html = "<object width=\"425\" height=\"350\">"
               + "<param name=\"movie\" value=\"http://www.youtube.com/v/"
               + arr[1] + "\"></param>"
               + "<param name=\"wmode\" value=\"transparent\"></param>"
               + "<embed src=\"http://www.youtube.com/v/" + arr[1]
               + "\" type=\"application/x-shockwave-flash\""
               + " wmode=\"transparent\" width=\"425\" height=\"350\">"
               + "</embed></object>"
          document.getElementById("urlbar").value = "http://www.youtube.com/"
                                                  + "watch?v/" +arr[1];
        }
        else {
          html = "<embed ";
          for (var j in event.target.attributes) {
            html += event.target.attributes.item(j).nodeName+"=\"";
            html += event.target.attributes.item(j).nodeValue+"\"";
          }
          html += ">"+event.target.innerHTML+"</embed>";
          var re1 = /(.+)&/;
          if (event.target.src.match(re1)) {
            var arr = re1.exec(event.target.src);
            event.target.src = arr[1];
            document.getElementById("urlbar").value = arr[1];
          }
        }
        html += "<p class=\"citation\"><cite cite=\"" + useUrl;
        html += "\"><a href=\"" + useUrl + "\">Embedded Video</a></cite></p>";
        mediaStream.show(event.target, mediaStream.mediaWebServices[i],
                         event.target.src, html, "video");
        break;
      }
    }
  }
}

mediaStream.mediaWebServices = new Array();
mediaStream.getFlockWebServices = function () {
  var CATMAN = Components.classes['@mozilla.org/categorymanager;1'].getService(Components.interfaces.nsICategoryManager);
  var e = CATMAN.enumerateCategory('flockWebService');
  while (e.hasMoreElements()) {
    try {
      var entry = e.getNext().QueryInterface(Components.interfaces.nsISupportsCString);
      if (!entry) { continue; }
      var contractID = CATMAN.getCategoryEntry('flockWebService', entry.data);
      var mediaService = Components.classes[contractID]
                                 .getService(Components.interfaces.flockIWebService);
      // We really only worry about media services so check those
      //flockIPhotoAPI
      if (mediaService instanceof Components.interfaces.flockIMediaWebService &&
          mediaService.handlesMediaStream() ) {
        mediaStream._logger.debug("Adding service: " + mediaService.title);
        mediaStream.mediaWebServices.push(mediaService);
      }
    } catch (e) {
      mediaStream._logger.debug("getFlockWebServices: " + e);
    }
  }
}


mediaStream.load = function() {
  // Use the flock logger
  mediaStream._logger.init('mediaStream');

  var stringBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"]
                                      .getService(Components.interfaces.nsIStringBundleService);
  mediaStream.bundle = stringBundleService.createBundle('chrome://flock/locale/services/mediaStream.properties');
  
  
  mediaStream.getFlockWebServices();

  // Monitor for network access (mainly location changes)
  gBrowser.addProgressListener(mediaStream.mWebProgressListener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
  
  // Monitor for mouse actions on the window so we can show/hide the overlay
  window.addEventListener('mouseover', mediaStream.imager, false);
  window.addEventListener('mouseout', mediaStream.imagerOut, false);
  window.addEventListener('mousemove', mediaStream.saveCords, false);
  
  // Monitor for high level actions so we can hide the overlay
  window.addEventListener('scroll', mediaStream.forceHide, false);
  window.addEventListener('resize', mediaStream.forceHide, false);
  
  // Monitor tab actions so we can hide when we need to
  var container = gBrowser.tabContainer;
  if (container) {
    container.addEventListener("TabClose", mediaStream.forceHide, false);
    container.addEventListener("TabOpen", mediaStream.forceHide, false);
    container.addEventListener("TabSelect", mediaStream.forceHide, false);
  }
}

mediaStream.unload = function() {
  gBrowser.removeProgressListener(mediaStream.mWebProgressListener);
  window.removeEventListener('mouseover', mediaStream.imager, false);
  window.removeEventListener('mouseout', mediaStream.imagerOut, false);
  window.removeEventListener('mousemove', mediaStream.saveCords, false);
  window.removeEventListener('scroll', mediaStream.forceHide, false);
  window.removeEventListener('resize', mediaStream.forceHide, false);
  
  var container = gBrowser.tabContainer;
  if (container) {
    container.removeEventListener("TabClose", mediaStream.forceHide, false);
    container.removeEventListener("TabOpen", mediaStream.forceHide, false);
    container.removeEventListener("TabSelect", mediaStream.forceHide, false);
  }
}

mediaStream.mWebProgressListener = {
  onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus) { },
  onProgressChange : function(aWebProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress) { },
  onStatusChange : function(aWebProgress, aRequest, aStatus, aMessage) { },
  onSecurityChange : function(aWebProgress, aRequest, aState) { },
  onLocationChange : function(aWebProgress, aRequest, aLocation) {
    // We need to hide the mediaStream box when we navigate away
    mediaStream.forceHide();
  }
}

/**
 Global listeners for the mediaStream system.
 Added an onload and onunload listener to setup our mediaStream
 listeners. These do not need to be removed since they will die with the browser.
 */
top.addEventListener('unload', mediaStream.unload, false);  // call unload to cleanup listeners
top.addEventListener('load', mediaStream.load, false);      // call load to setup listeners and register the handlers

